//============================================================================
// FILE: ContinuousProcess.h
//
// TYPE: C++CLASS
// DESCRIPTION: entête de la classe générique des processus continus
// 
// AUTHORS: Rellier (INRA/UBIA, Toulouse)
//=X===============X=============================X============================

#ifndef _CONTINUOUSPROCESS_HH
#define _CONTINUOUSPROCESS_HH

// SYSTEM INCLUDES
//
//#include <>                                    //

// PROJECT INCLUDES
//
#include "Process.h" 
 
          /**@name CDPFORTEXLa classe des processus continus
\begin{itemize} \item Directive d'inclusion : 
[P]
\#include ContinuousProcess.h
[/P]
\end{itemize}

Les sous-classes de processus continus propres au système simulé sont déclarées et réalisées dans les fichiers typiquement nommés [F]UserContinuousProcess.h[/F] et [F]UserContinuousProcess.cc[/F].\\

On indique en préambule qu'un processus continu ne peut être contrôlé que par les types d'action (directives) INIT, STOP ou PROCEED.\\ 
Pour chaque classe propre à son application, l'utilisateur doit essentiellement coder quatre méthodes (en plus de l'éventuelle fonction de précondition, décrite dans la classe [C]Process[/C]) :
\begin{enumerate}
\item Une fonction (nommée par exemple et arbitrairement {\em MyProcess\_Initialize}) qui code l'initialisation des descripteurs des entités touchées par le processus. Le moteur de simulation, lorsqu'il rencontre dans un événement une directive INIT associée à un processus continu, invoque cette fonction d'initialisation.
\item Une fonction qui code l'avancée du processus pour une période de un {\em pas}, c'est-à-dire les changements d'état des entités touchées par le processus. Le moteur de simulation, lorsqu'il rencontre dans un événement une directive PROCEED associée à un processus continu, invoque la méthode prédéfinie [Y]Proceed[/Y] de la classe [C]ContinuousProcess[/C]. Cette méthode fait avancer le processus {\em pas} par {\em pas} en enchaînant des appels à la fonction de changement d'état. 
\noLF
Cette avancée est entretenue tant qu'il n'est pas nécessaire de l'interrompre, du fait de la présence dans l'agenda d'un événement prioritaire (au sens de la fonction attachée à la simulation par [Y]AssignBeforeInAgendaMethod[/Y]\index{AssignBeforeInAgendaMethod}). Dans cette situation le moteur de simulation :
\begin{enumerate}
\item cesse d'invoquer dans [Y]Proceed[/Y] la fonction de changement d'état,
\item place dans l'agenda un nouvel événement de classe [C]ProceedFromCurrentState[/C] avec pour seule conséquence un couple constitué du processus interrompu et d'une directive PROCEED. La date de ce nouvel événement est la date courante, c'est-à-dire celle à laquelle le processus a été interrompu et à partir de laquelle il doit être poursuivi. L'événement d'origine est détruit avec ses méthodes mais sans ses processus.
\end{enumerate}

Il est important de noter que le {\em pas} est la durée d'une période pendant laquelle il est impossible d'interrompre la procédure de changement d'état du système. En conséquence, l'évolution de l'état à l'intérieur (strictement) de ce laps de temps ne peut servir de donnée de contrôle pour d'autres processus, ni être contrôlée par d'autres processus. Seules les valeurs d'état en début et en fin du laps de temps pourront servir au contrôle du système. L'utilisateur doit établir les valeurs de {\em pas} en tenant compte de cela, notamment lorsque la bonne {\em synchronisation}\index{synchronisation} des processus régissant le système est exigée. Bien entendu, la fonction d'avancée du processus sur un {\em pas} peut être codée comme une boucle, avec incrémentation d'un ``pas'', mais ce dernier est un ``pas'' interne, sans rapport avec le {\em pas} du processus (qu'on peut maintenant qualifier d'externe).\\[1mm]
Noter aussi que si un événement doit intervenir, non pas au moment même où est invoquée la fonction d'avancée d'un processus continu, mais pendant son exécution, alors (i) la fonction est effectivement invoquée une dernière fois, (ii) un événement de reprise du processus continu est programmé à la date d'effet du changement, puis enfin (iii) l'événement qui aurait dû intervenir pendant le changement d'état est déclenché (avec un retard minimal, inférieur ou égal au {\em pas} du processus continu).

\item Une fonction qui code l'arrêt du processus, invoquée par le moteur de simulation, lorsqu'il rencontre dans un événement une directive STOP associée à un processus continu. Elle peut aussi être invoquée explicitement dans une partie quelconque du code écrit par l'utilisateur.

\item Le constructeur de la classe, dans lequel l'utilisateur doit appeler le constructeur de la classe de base, avec le symbole (et éventuellement le nom de classe) qu'il associe à la classe en argument. Par exemple :
\begin{verbatim}
[P]MyProcess::MyProcess() : ContinuousProcess(MY_SYMBOL) { ... };[/P]
\end{verbatim}

Dans ce constructeur ou en dehors, l'utilisateur doit impérativement doter le processus des fonctions d'initialisation, d'avancée de un {\em pas} et d'arrêt. Par exemple :
\begin{verbatim}
[P]  SetInitialize(MyProcess_Initialize);
  SetGoOneStepForward(MyProcess_GoOneStepForward);
  SetStop(MyProcess_Stop);[/P]
\end{verbatim}

\end{enumerate}
*/
//@{
        ///
class ContinuousProcess : public Process
{
public: 
  /**@name LIFECYCLE */
//@{  
          // Juste pour le besoin de la swiguisation sous Linux 2.4.25 (Carré)
  ContinuousProcess();
          // Constructeur de base sans méthode. Appel exclusif dans RawCopy avec l'argument 1. Les méthodes sont affectées dans RawCopy
  //ContinuousProcess(bool); // _no swig_
          /// Un des deux constructeurs qui doivent être appelés par les classes définies par l'utilisateur. Affecte le type CONTINUOUS\_PROCESS, la valeur 1 au {\em pas} du processus (modifiable par appel de la méthode [Y]Step(int)[/Y] et le symbole de classe donné en argument. Utiliser la méthode [Y]ClassName[/Y] pour affecter un nom de classe.
   ContinuousProcess(UserProcessId); // _no swig_                            
          /// Un des deux constructeurs qui doivent être appelés par les classes définies par l'utilisateur. Mêmes affectations que ci-dessus, plus le nom de classe donné en second argument.\label{affectNomCProcess}
   ContinuousProcess(UserProcessId, char*); // _no swig_                  
          /// Constructeur par copie profonde, y compris des méthodes.  Invoque le constructeur par copie de la classe-mère.
   ContinuousProcess(const ContinuousProcess&); // _no swig_  JPR 060328
                   
          // Constructeur par copie superficielle, c'est-à-dire sans copie des méthodes. La nouvelle instance pointe sur les mêmes méthodes dont les corps codent l'initialisation du processus et son avancée pour une période de un {\em pas}. L'entité sur laquelle le processus copié opère un changement d'état est la m\^eme que pour le processus original.\index{méthodes!RawCopy - ContinuousProcess}    
   //ContinuousProcess* RawCopy(); // obsolète 060328
            
          /// Destructeur superficiel. Ne détruit pas l'entité sur laquelle le processus opère des changements d'état, ni les instances de méthodes.
   ~ContinuousProcess(); // _no swig_                     
          /// Méthode de destruction. Ne détruit pas l'entité sur laquelle le processus opère des changements d'état. Détruit les instances de [C]Method[/C] attachées au processus (précondition, initialisation, avancée, arrêt).
   void DeleteInDepth(); // _no swig_
//@} 
  

/**@name OPERATORS */
//@{
         /// Opérateur d'affectation. Invoque les opérateurs d'affectation des méthodes. Renvoie une référence à l'entité d'origine.
   const
   ContinuousProcess&      operator=(const ContinuousProcess&);  
//@}       


/**@name [SC]Le pas du processus[/SC] */
//@{ 
         /// Affecte la valeur donnée en argument au {\em pas} du processus. \index{méthodes!Step}
     void     Step(int); // _no swig_ 
     void     Pas(int); // _no swig_  // obsolète (maintenu pour compatibilité)
         /// Renvoie la valeur du {\em pas} du processus.
   int        Step() const;
   int        Pas() const;  // obsolète (maintenu pour compatibilité)

         //
   void      IsReadSDF(Boolean val);
         //
   Boolean      IsReadSDF() const;
//@}
 

/**@name [SC]Programmation de l'exécution[/SC] */
//@{

         // Méthode prédéfinie utilisée exclusivement par le moteur de simulation pour opérer l'effet du processus sur un {\em pas} de temps. L'argument est l'événement dont on exécute une directive. \index{méthodes!Proceed}
   void               Proceed(Event*);  
         /// Insère dans l'agenda un événement d'initialisation du processus récepteur de ce message, à la date passée en argument. La priorité de l'événement est 0. Renvoie TRUE seulement si l'événément a pu être inséré. Désalloue l'événement instancié dans le cas contraire. Le code appelant doit se charger de l'éventuelle désallocation du processus.\index{méthodes!PostInitialisationEvent}
   Boolean            PostInitialisationEvent(int); // _no swig_  
         /// Même effet que la méthode précédente. La priorité de l'événement est passée en second argument.
   Boolean            PostInitialisationEvent(int, int);  

         /// Insère dans l'agenda un événement d'initialisation et de continuation immédiate du processus récepteur de ce message, à la date passée en argument. La priorité de l'événement est 0. Renvoie TRUE seulement si l'événément a pu être inséré. Désalloue l'événement instancié dans le cas contraire. Le code appelant doit se charger de l'éventuelle désallocation du processus.\index{méthodes!PostInitAndProceedEvent}
   Boolean            PostInitAndProceedEvent(int); // _no swig_  
         /// Même effet que la méthode précédente. La priorité de l'événement est passée en second argument.
   Boolean            PostInitAndProceedEvent(int, int);   

         /// Insère dans l'agenda deux événements : l'un d'initialisation à la date passée en premier argument, et l'autre de continuation du processus récepteur de ce message, à la date passée en second argument. La priorité de l'événement est 0. Renvoie TRUE seulement si les deux événéments ont pu être insérés. Désalloue la ou les deux événements instanciés dans le cas contraire. Le code appelant doit se charger de l'éventuelle désallocation du processus.\index{méthodes!PostInitThenProceedEvent}
   Boolean            PostInitThenProceedEvent(int, int); // _no swig_  
         /// Même effet que la méthode précédente. La priorité de l'événement est passée en troisième argument.
   Boolean            PostInitThenProceedEvent(int, int, int);  

         /// Insère dans l'agenda un événement de continuation du processus récepteur de ce message (à partir de son état courant), à la date passée en argument. La priorité de l'événement est 0. Désalloue l'événement instancié dans le cas contraire. Le code appelant doit se charger de l'éventuelle désallocation du processus.\index{méthodes!PostProceedEvent}
   Boolean            PostProceedEvent(int); // _no swig_  
         /// Même effet que la méthode précédente. La priorité de l'événement est passée en second argument.   
   Boolean            PostProceedEvent(int, int); 
//@}  


/**@name [SC]Initialisation, avanc\'ee, arr\^et[/SC]
\\

Le message : [P]<processus>->SetInitialize({\em f1});[/P] affecte un corps {\small {\tt f1}} à l'instance de [C]Method[/C] (précisément instance de [C]InitializeProcess[/C]) dont le symbole de classe est INITIALIZE\_PROCESS. Ce corps réalise l'initialisation du processus.\\

Le message : [P]<processus>->SetGoOneStepForward({\em f2});[/P] affecte un corps {\em f2} à l'instance de [C]Method[/C] (précisément instance de [C]GoOneStepForwardProcess[/C]) dont le symbole de classe est GOONESTEPFORWARD\_PROCESS. Ce corps, invoqué répétitivement jusqu'à interruption dans la méthode prédéfinie [Y]Proceed[/Y], réalise l'avancée de l'effet du processus pour un {\em pas} de temps.\\

Il y a trois manières d'arrêter un processus continu :
\begin{itemize} 
\item insérer dans l'agenda un événement avec la directive STOP, laquelle provoque l'invocation [P]ExecVoidMethod(STOP\_PROCESS);[/P] sur le processus en jeu. Il faut alors avoir assigné un corps à cette méthode prédéfinie ;
\item invoquer directement [P]ExecVoidMethod(STOP\_PROCESS);[/P], après avoir associé au processus (par [Y]SetStop[/Y]) une fonction "utilisateur" contenant l'appel [P]ToBeStopped(TRUE);[/P]. Il faut alors avoir assigné un corps à cette méthode prédéfinie ;
\item invoquer directement la méthode [Y]ToBeStopped[/Y].
\end{itemize}

Le message : [P]<processus>}->SetStop({\em f3});[/P] affecte un corps {\em f3} à l'instance de [C]Method[/C] (précisément instance de [C]StopProcess[/C]) dont le symbole de classe est STOP\_PROCESS. Ce corps réalise l'arrêt du processus. Par exemple, si l'arrêt du processus {\em pP} est demandé à l'intérieur de la fonction qui code l'avancée sur un {\em pas}, l'appel sera écrit [/P]pP->ExecVoidMethod(STOP\_PROCESS);[/P] Normalement, {\small {\tt f3}} contient une invocation de [/P]ToBeStopped(TRUE);[/P] sur le processus en jeu.\\

Par une convention propre à \D, chaque corps prend comme seul argument l'instance de [C]Method[/C] dont il est le corps. Le processus en jeu est donc accessible par une invocation de [Y]DescribedContinuousProcess[/Y] sur la méthode. Dans chacun de ces corps, on peut donc accéder à une entité particulière, celle renvoyée par l'invocation de [Y]ProcessedEntity[/Y] sur le processus en jeu. C'est typiquement, mais non obligatoirement, celle dont le processus change les variables d'état. Par exemple, {\small {\tt f3}} peut être :
\begin{verbatim}
[P]void MyProcessClass_Fct3(ProcessMethod* pM) {
  ContinuousProcess* pP = pM->DescribedContinuousProcess();
  Entity* pE = pP->ProcessedEntity();
  pP->ToBeStopped(TRUE);
  ...
}[/P]
\end{verbatim}
*/

//@{
         /// Dote le processus d'un corps de méthode qui code son initialisation (c'est-à-dire l'initialisation des valeurs des variables d'état qu'il contrôle). \index{méthodes!SetInitialize}
   void            SetInitialize(void (* function)(ProcessMethod*)); // _no swig_
  
         /// Dote le processus d'un corps de méthode qui code son avancée pour une période de un {\em pas} (c'est-à-dire les changements de valeur des variables d'état qu'il provoque), à partir de la dernière date où l'état a été calculé jusqu'à la même date augmentée du {\em pas} du processus. \index{méthodes!SetGoOneStepForward}
   void            SetGoOneStepForward(void (* function)(ProcessMethod* pM)); // _no swig_

         /// Dote le processus d'un corps de méthode qui code son arrêt. La fonction peut ne contenir qu'un message [P]ToBeStopped(TRUE);[/P] adressé au processus placé en argument, ou bien ce message plus autre chose. \index{méthodes!SetStop}
   void            SetStop(void (* function)(ProcessMethod*)); // _no swig_
         /// Détermine si le processus doit être arrêté (avec TRUE en argument). \index{méthodes!ToBeStopped}
   void            ToBeStopped(Boolean); // _no swig_  
         /// Renvoie TRUE si le processus doit être arrêté, FALSE (par défaut) sinon.
   Boolean         ToBeStopped() const;
//@} 

// INQUIRY  

   Boolean       CheckForInterruption(int, Event*);// (int horloge) _no swig_ 
   
    // comptage
    static int   getNbConst() {return _nbConst;}
    static int   getNbDest() {return _nbDest;}  

protected:

   int static   _nbConst;
   int static   _nbDest; 

   int             mStep;
   Boolean         mToBeStopped;

   Boolean         mIsReadSDF;

private:
};  // _end class_

//@} // fin chapitre

// INLINE METHODS


// EXTERNAL REFERENCES

#endif  // _CONTINUOUSPROCESS_HH
