//============================================================================
// FILE: CoreMethod.h
//
// TYPE: C++CLASS 
// DESCRIPTION: entête des méthodes "moteur"
// 
// AUTHORS: Rellier (INRA/UBIA, Toulouse)
//=X===============X=============================X============================

#ifndef _COREMETHOD_HH
#define _COREMETHOD_HH

// SYSTEM INCLUDES
//
//#include <>                                    //

// PROJECT INCLUDES
//
#include "Method.h"

/**@name CDPFORTEXMéthodes prédéfinies
\begin{itemize} \item Directive d'inclusion :
[P]
\#include CoreMethod.h
[/P]
\end{itemize}

C'est normalement l'utilisateur qui crée des classes spécialisées de méthodes, descendantes de la classe mère [C]Method[/C], en fonction de son domaine d'application. Cependant, le développement de la bibliothèque \Db elle-même a occasionné, pour son besoin propre, la prédéfinition de certaines méthodes, sous-classes elles-aussi de [C]Method[/C].

De manière générale le {\em corps} de ces méthodes n'est pas codé dans \D, puisqu'il représente une partie de la connaissance propre au domaine d'application. Par exemple, la méthode prédéfinie d'exécution d'un processus ponctuel en général recevra un {\em corps} spécifique au domaine ; il ne peut pas être imaginé dans \D. Il revient donc à l'utilisateur d'écrire ce {\em corps}, puis de l'attacher à la méthode prédéfinie (par exemple par un [Y]AssignBody[/Y] adressé à la méthode). Par contre, la place de l'invocation de la méthode prédéfinie dans l'algorithme de simulation, et la manière d'exécuter son {\em corps} relèvent d'un mécanisme général codé au niveau de \D.

{\large {\tt Les méthodes prédéfinies}}

\begin{itemize}

\item pour les processus, il s'agit de l'initialisation, de la poursuite et de l'arrêt d'un processus continu, de l'exécution d'un processus discret, et enfin de la précondition de tous les processus.

\item pour les événements, il s'agit des méthodes qui définissent la post-conséquence d'un événement, qui auto-génèrent un événement à partir de celui en cours d'exécution et enfin qui complètent la spécification d'un événement déjà auto-généré (mise à jour complète ou spécification de date seulement).

\item pour les démons, il s'agit des méthodes qui définissent la réaction à déclencher lors d'événéments d'accès ou de modification des valeurs des descripteurs auxquels les démons sont attachés.

\item pour les spécifications d'ensembles d'entités, il s'agit des méthodes de sélection et d'instantiation des entités, ainsi que de la méthode de calcul de l'effectif.

\item pour les spécifications de domaines de valeurs, il s'agit de la méthode qui détermine si une entité satisfait la spécification.
\end{itemize}

\Db peut proposer une version par défaut du {\em corps} de la méthode, lorsqu'il est s'appliquer de manière générale. C'est le cas de la méthode de génération du prochain événement autogénéré : le {\em corps} gère l'insertion dans l'agenda d'événements, à partir des délais spécifiés par l'utilisateur pour l'intervalle entre deux événements. 
*/ 
//@{

/**@name  Méthodes prédéfinies de processus */ 
//@{

// X///////////////X/////////////////////////////X////////////////////////////

/**@name Exécution d'un processus discret 
Le corps, affecté par [Y]SetExec[/Y], réalise l'effet du processus sur les composants du système touchés par le processus.\index{ExecProcess}.\label{defExecProcess}

\begin{verbatim}
[P]void myDiscreteProcess_execProcess_Body(ProcessMethod* pM) {
  DiscreteProcess* pDP = pM->DescribedDiscreteProcess();
  Entity* pE = pDP->ProcessedEntity();
  ... // l'effet du processus
}
MyDiscreteProcess::MyDiscreteProcess() 
   : DiscreteProcess(MY_DISCRETE_PROCESS, "myDiscreteProcess") {
  ProcessedEntityClassId(BD_ENTITY);
  SetExec(myDiscreteProcess_execProcess_Body);
};[/P]
\end{verbatim}

Le constructeur [Y]ExecProcess()[/Y] affecte le symbole de classe EXEC_PROCESS et le nom de classe "execProcess". Ainsi, on peut écrire :
\begin{verbatim}
[P]int main(...) {
  ...
  DiscreteProcess* pDP = new MyDiscreteProcess();

  // le code ci-dessous ...
  Method* pM = pDP->GetMethod(EXEC_PROCESS);
  ((VoidProcessMethod*)pM)->Body((ProcessMethod*)pM);

  // ... est équivalent à celui-ci
  pDP->ExecVoidMethod(EXEC_PROCESS);
  ...
}[/P]
\end{verbatim}
*/
//@{
         ///
class ExecProcess : public VoidProcessMethod
{
public:
   ExecProcess();           
 }; // _end class_

//@} // fin classe

// X///////////////X/////////////////////////////X////////////////////////////

/**@name Initialisation d'un processus continu
Le corps, affecté par [Y]SetInitialize[/Y], établit l'état initial des composants du système touchés par le processus.\index{InitializeProcess}.

\begin{verbatim}
[P]void myContinuousProcess_initializeProcess_Body(ProcessMethod* pM) {
  ContinuousProcess* pCP = pM->DescribedContinuousProcess();
  Entity* pE = pCP->ProcessedEntity();
  ... // l'initialisation de l'état du système touché par le processus
}
MyContinuousProcess::MyContinuousProcess() 
   : ContinuousProcess(MY_CONTINUOUS_PROCESS, "myContinuousProcess") {
  Step(24);
  ProcessedEntityClassId(BD_ENTITY);
  SetInitialize(myContinuousProcess_initializeProcess_Body);
};[/P]
\end{verbatim}

 Le constructeur [Y]InitializeProcess()[/Y] affecte le symbole de classe INITIALIZE_PROCESS et le nom de classe "initializeProcess". Ainsi, on peut écrire :
\begin{verbatim}
[P]int main(...) {
  ...
  ContinuousProcess* pCP = new MyContinuousProcess();

  // le code ci-dessous ...
  Method* pM = pCP->GetMethod(INITIALIZE_PROCESS);
  ((VoidProcessMethod*)pM)->Body((ProcessMethod*)pM);

  // ... est équivalent à celui-ci
  pCP->ExecVoidMethod(INITIALIZE_PROCESS);
  ...
}[/P]
*/
//@{
         ///
class InitializeProcess : public VoidProcessMethod
{
public:
   InitializeProcess();           
 }; // _end class_

//@} // fin classe

// X///////////////X/////////////////////////////X////////////////////////////

/**@name Continuation d'un processus continu
Le corps, affecté par [Y]SetGoOneStepForward[/Y], code l'avancée pour un pas de l'effet du processus sur les composants du système touchés par le processus.\index{GoOneStepForwardProcess}

\begin{verbatim}
[P]void myContinuousProcess_goOneStepForwardProcess_Body(ProcessMethod* pM) {
  ContinuousProcess* pCP = pM->DescribedContinuousProcess();
  Entity* pE = pCP->ProcessedEntity();
  ... // l'évolution en un pas de l'état du système touché par le processus
}
MyContinuousProcess::MyContinuousProcess() 
   : ContinuousProcess(MY_CONTINUOUS_PROCESS, "myContinuousProcess") {
  Step(24);
  ProcessedEntityClassId(BD_ENTITY);
  SetGoOneStepForward(myContinuousProcess_goOneStepForwardProcess_Body);
};[/P]
\end{verbatim}

Le constructeur [Y]GoOneStepForwardProcess()[/Y] affecte le symbole de classe GOONESTEPFORWARD_PROCESS et le nom de classe "goOneStepForwardProcess". Ainsi, on peut écrire :
\begin{verbatim}
[P]int main(...) {
  ...
  ContinuousProcess* pCP = new MyContinuousProcess();

  // le code ci-dessous ...
  Method* pM = pCP->GetMethod(GOONESTEPFORWARD_PROCESS);
  ((VoidProcessMethod*)pM)->Body((ProcessMethod*)pM);

  // ... est équivalent à celui-ci
  pCP->ExecVoidMethod(GOONESTEPFORWARD_PROCESS);
  ...
}[/P]
\end{verbatim}
*/
//@{
         ///
class GoOneStepForwardProcess : public VoidProcessMethod
{
public:
   GoOneStepForwardProcess();           
 }; // _end class_

//@} // fin classe

// X///////////////X/////////////////////////////X////////////////////////////

/**@name Arrêt d'un processus continu
 Le corps, affecté par [Y]SetStop[/Y], code l'arrêt du processus continu.\index{StopProcess}

\begin{verbatim}
[P]void myContinuousProcess_stopProcess_Body(ProcessMethod* pM) {
  ContinuousProcess* pCP = pM->DescribedContinuousProcess();
  Entity* pE = pCP->ProcessedEntity();
  ... // l'évolution en un pas de l'état du système touché par le processus
}
MyContinuousProcess::MyContinuousProcess() 
   : ContinuousProcess(MY_CONTINUOUS_PROCESS, "myContinuousProcess") {
  Step(24);
  ProcessedEntityClassId(BD_ENTITY);
  SetGoOneStepForward(myContinuousProcess_stopProcess_Body);
};[/P]
\end{verbatim}

Le constructeur [Y]StopProcess()[/Y] affecte le symbole de classe STOP_PROCESS et le nom de classe "stopProcess". Ainsi, on peut écrire :
\begin{verbatim}
[P]int main(...) {
  ...
  ContinuousProcess* pCP = new MyContinuousProcess();

  // le code ci-dessous ...
  Method* pM = pCP->GetMethod(STOP_PROCESS);
  ((VoidProcessMethod*)pM)->Body((ProcessMethod*)pM);

  // ... est équivalent à celui-ci
  pCP->ExecVoidMethod(STOP_PROCESS);
  ...
}[/P]
\end{verbatim}

On rappelle ici que la méthode [C]StopProcess[/C] est automatiquement invoquée lorsque le moteur de simulation rencontre un événement avec une directive STOP sur un processeur continu. Voir à la page 'La classe des processus continus' les différentes manières d'arrêter un processus continu.
 */
//@{
         ///
class StopProcess : public VoidProcessMethod
{
public:
   StopProcess();           
 }; // _end class_

//@} // fin classe

// X///////////////X/////////////////////////////X////////////////////////////

/**@name Exploitation des données lues sur un fichier séquentiel
 Le corps, affecté par [Y]SetPostRead[/Y], code l'exploitation des données lues sur un fichier séquentiel.\index{PostRead}

\begin{verbatim}
[P]void lectureFichierClimat_initializeProcesss_Body(ProcessMethod* pM) {
  // voir un exemple de contenu à la page 
  // 'Le processus continus de lecture d'un fichier de données séquentielles'
}
void lectureFichierClimat_postReadProcesss_Body(ProcessMethod* pM) {
  // voir un exemple de contenu à la page 
  // 'Le processus continus de lecture d'un fichier de données séquentielles'
}

LectureFichierClimat::LectureFichierClimat() 
   : ReadSequentialDataFileProcess(LECTURE_FICHIER_CLIMAT, "lectureFichierClimat") {
  ProcessedEntityClassId(WEATHER);
  Step(1);
  FileClassId(FICHIER_CLIMAT);
  SetInitialize(lectureFichierClimat_initializeProcess_Body);
  SetPostRead(lectureFichierClimat_postReadProcess_Body);
};[/P]
\end{verbatim}

Le constructeur [Y]PostRead()[/Y] affecte le symbole de classe POST_READ et le nom de classe "postRead". Ainsi, on peut écrire :
\begin{verbatim}
[P]int main(...) {
  ...
  LectureFichierClimat* pCP = new LectureFichierClimat();

  // le code ci-dessous ...
  Method* pM = pCP->GetMethod(POST_READ);
  ((VoidProcessMethod*)pM)->Body((ProcessMethod*)pM);

  // ... est équivalent à celui-ci
  pCP->ExecVoidMethod(POST_READ);
  ...
}[/P]
\end{verbatim}
 */
//@{
         ///
class PostRead : public VoidProcessMethod
{
public:
   PostRead();           
 }; // _end class_

//@} // fin classe

// X///////////////X/////////////////////////////X////////////////////////////

/**@name Précondition d'un processus
Le corps, affecté par [Y]SetPrecondition[/Y], code la précondition du processus.\index{PreconditionProcess}

\begin{verbatim}
[P]int myContinuousProcess_processPrecondition_Body(ProcessMethod*) pM) {
  result = 1;
  ContinuousProcess* pCP = pM->DescribedContinuousProcess();
  Entity* pE = pCP->ProcessedEntity();
  ... // valuation de result à 1 (précondition satisfaite) ou à 0
  return result;
}
MyContinuousProcess::MyContinuousProcess() 
   : ContinuousProcess(MY_CONTINUOUS_PROCESS, "myContinuousProcess") {
  Step(24);
  ProcessedEntityClassId(BD_ENTITY);
  SetPrecondition(myContinuousProcess_processPrecondition_Body);
};[/P]
\end{verbatim}

Le constructeur [Y]ProcessPrecondition()[/Y] affecte le symbole de classe PROCESS_PRECONDITION et le nom de classe "processPrecondition". Ainsi, on peut écrire :
\begin{verbatim}
[P]int main(...) {
  ...
  ContinuousProcess* pCP = new MyContinuousProcess();
  int preConditionValue;

  // le code ci-dessous ...
  Method* pM = pCP->GetMethod(PROCESS_PRECONDITION );
  preConditionValue = ((IntProcessMethod*)pM)->Body((ProcessMethod*)pM);

  // ... est équivalent à celui-ci
  preConditionValue = pCP->ExecIntMethod(PROCESS_PRECONDITION);
  ...
}[/P]

*/
//@{
         ///
class ProcessPrecondition : public IntProcessMethod
{
public:
   ProcessPrecondition();           
 }; // _end class_

//@} // fin classe

// X///////////////X/////////////////////////////X////////////////////////////

/**@name Délai de ré-examen d'un processus
Le corps, affecté par [Y]SetPostponementDelay[/Y], renvoie le délai de ré-examen d'un processus quand il a été retardé (préconditions non satisfaites).

\begin{verbatim}
[P]int myDiscreteProcess_processPostponementDelay_Body(ProcessMethod*) pM) {
  result = 1;
  DiscreteProcess* pDP = pM->DescribedDiscreteProcess();
  Entity* pE = pDP->ProcessedEntity();
  ... // valuation de result avec le délai requis
  return result;
}
MyDiscreteProcess::MyDiscreteProcess() 
   : DiscreteProcess(MY_DISCRETE_PROCESS, "myDiscreteProcess") {
  Step(24);
  ProcessedEntityClassId(BD_ENTITY);
  SetPostponementDelay(myDiscreteProcess_processPostponementDelay_Body);
};[/P]
\end{verbatim}

Le constructeur [Y]ProcessPostponementDelay()[/Y] affecte le symbole de classe PROCESS_POSTPONEMENT_DELAY et le nom de classe "processPostponementDelay". Ainsi, on peut écrire :
\begin{verbatim}
[P]int main(...) {
  ...
  ContinuousProcess* pCP = new MyContinuousProcess();
  int postponementValue;

  // le code ci-dessous ...
  Method* pM = pCP->GetMethod(PROCESS_POSTPONEMENT_DELAY );
  postponementValue = ((IntProcessMethod*)pM)->Body((ProcessMethod*)pM);

  // ... est équivalent à celui-ci
  postponementValue = pCP->ExecIntMethod(PROCESS_POSTPONEMENT_DELAY);
  ...
}[/P]
\end{verbatim}
*/
//@{
         ///
class ProcessPostponementDelay : public IntProcessMethod
{
public:
   ProcessPostponementDelay();           
 }; // _end class_

//@} // fin classe

//@}  // fin méthodes processus




// X///////////////X/////////////////////////////X////////////////////////////

/**@name Méthodes prédéfinies d'événements */ 
//@{

// X///////////////X/////////////////////////////X////////////////////////////

/**@name Définition de la post-conséquence d'un événement
Le corps, affecté par [Y]SetPostConsequence[/Y], établit la post-conséquence d'un événement.\index{EventPostConsequence}

\begin{verbatim}
[P]void myEvent_eventPostConsequence_Body(EventMethod* pM) {
  Event* pEv = pM->DescribedEvent();
  ... // ce qui doit être fait après le traitement réussi des directives sur les processus
}
MyEvent::MyEvent() 
   : Event(MY_EVENT, "myEvent") {
  ...
  SetPostConsequence(myEvent_eventPostConsequence_Body);
  ...
};[/P]
\end{verbatim}

Le constructeur [Y]EventPostConsequence()[/Y] affecte le symbole de classe EVENT_POSTCONSEQUENCE et le nom de classe "eventPostConsequence". Ainsi, on peut écrire :
\begin{verbatim}
[P]int main(...) {
  ...
  Event* pEv = new MyEvent();

  // le code ci-dessous ...
  Method* pM = pEv->GetMethod(EVENT_POSTCONSEQUENCE);
  ((VoidEventMethod*)pM)->Body((EventMethod*)pM);

  // ... est équivalent à celui-ci
  pEv->ExecVoidMethod(EVENT_POSTCONSEQUENCE);
  ...
}[/P]
\end{verbatim}
*/
//@{
         ///
class EventPostConsequence : public VoidEventMethod
{
public:
   EventPostConsequence();           
 }; // _end class_

//@} // fin classe

// X///////////////X/////////////////////////////X////////////////////////////

/**@name Mise à jour complète d'un événement auto-généré
Le corps, affecté par [Y]SetUpdateNextEvent[/Y], donne des valeurs spécifiques à un événement qui vient d'être auto-généré.\index{UpdateNextEvent}.

\begin{verbatim}
[P]void myEvent_updateNextEvent_Body(EventMethod* pM) {
  Event* pPreviousEvent = pM->GetEventArgValue(PREVIOUS_EVENT_ARGUMENT);
  Event* pEventToUpdate = pPreviousEvent->NextEvent();
  ... // les ajustements définissant le prochain événement (pEventToUpdate)
}
MyEvent::MyEvent() 
   : Event(MY_EVENT, "myEvent") {
  ...
  SetUpdateNextEvent(myEvent_updateNextEvent_Body);
  ...
};[/P]
\end{verbatim}

Le constructeur [Y]UpdateNextEvent()[/Y] affecte le symbole de classe UPDATE_NEXT_EVENT et le nom de classe "updateNextEvent". Ainsi, on peut écrire :
\begin{verbatim}
[P]int main(...) {
  ...
  Event* pEv = new MyEvent();

  // le code ci-dessous ...
  Method* pM = pEv->GetMethod(UPDATE_NEXTEVENT);
  pM->SetArgValue(PREVIOUS_EVENT_ARGUMENT, pEv);
  ((EventVoidMethod*)pM)->Body((EventMethod*)pM);

  // ... est équivalent à celui-ci
  pEv->SetEventArgValue(UPDATE_NEXTEVENT, PREVIOUS_EVENT_ARGUMENT, pEv);
  pEv->ExecVoidMethod(UPDATE_NEXTEVENT);
  ...
}[/P]
\end{verbatim}
 */
//@{
         ///
class UpdateNextEvent : public VoidEventMethod
{
public:
   UpdateNextEvent();           
 }; // _end class_

//@} // fin classe

// X///////////////X/////////////////////////////X////////////////////////////

/**@name Auto-génération d'un événement
Le corps, affecté par [Y]SetGenerateNextEvent[/Y], génére un nouvel événement à partir de celui qui est en cours d'exécution, avant de l'insérer dans l'agenda. Le nouvel événement est d'une classe différente ou de la même classe, selon qu'une invocation de [Y]SetNextEventClassName[/Y] a été faite ou non lors de la construction. Par défaut d'un tel appel, les deux noms de classe sont les mêmes.\index{GenerateNextEvent}

\begin{verbatim}
[P]bool pingEvent_generateNextEvent_Body(EventMethod* pM) {
  bool succeed = TRUE;
  Event* pCurrentEvent = pM->DescribedEvent();
  ...
  Event* pNextEvent = new PongEvent();
  ...
  pCurrentEvent->SetNextEvent(pNextEvent);
  ...
  return succeed;
}
PingEvent::PingEvent() 
   : Event(PING_EVENT, "pingEvent") {
  ...
  SetGenerateNextEvent(pingEvent_generateNextEvent_Body);
  ...
};[/P]
\end{verbatim}

Le constructeur [Y]GenerateNextEvent()[/Y] affecte le symbole de classe GENERATE_NEXTEVENT et le nom de classe "generateNextEvent". Ainsi, on peut écrire :
\begin{verbatim}
[P]int main(...) {
  ...
  Event* pEv = new PingEvent();
  bool succeed;

  // le code ci-dessous ...
  Method* pM = pEv->GetMethod(GENERATE_NEXTEVENT);
  succeed = ((EventPredicate*)pM)->Body((EventMethod*)pM);

  // ... est équivalent à celui-ci
  succeed = pEv->ExecPredicate(GENERATE_NEXTEVENT);
  ...
}[/P]
\end{verbatim}
 */
//@{
         ///
bool standardEvent_generateNextEvent_Body(EventMethod*);
         ///
class GenerateNextEvent : public EventPredicate
{
public:
   GenerateNextEvent();           
 }; // _end class_

//@} // fin classe

// X///////////////X/////////////////////////////X////////////////////////////

/**@name Détermination (non aléatoire) de la date d'un événement auto-généré
Le corps, affecté par [Y]SetDefineNextEventClockTime[/Y], établit et renvoie la date d'un événement qui vient d'être auto-généré.\index{DefineNextEventClockTime}

\begin{verbatim}
[P]int myEvent_defineNextEventClockTime_Body(EventMethod* pM) {
  Event* pPreviousEvent = pM->GetEventArgValue(PREVIOUS_EVENT_ARGUMENT);
  Event* pEventToDate = pPreviousEvent->NextEvent();
  int nextEventClock;
  ... // les ajustements définissant la date du prochain événement (pEventToDate)
  pEventToDate->SetOccurrenceClockTime(nextEventClock);
  ...
  return nextEventClock;
}
MyEvent::MyEvent() 
   : Event(MY_EVENT, "myEvent") {
  ...
  SetDefineNextEventClockTime(myEvent_defineNextEventClockTime_Body);
  ...
};[/P]
\end{verbatim}

Le constructeur [Y]DefineNextEventClockTime()[/Y] affecte le symbole de classe DEFINE_NEXTEVENTCLOCKTIME et le nom de classe "defineNextEventClockTime". Ainsi, on peut écrire :
\begin{verbatim}
[P]int main(...) {
  ...
  Event* pEv = new MyEvent();
  int nextEventClock;
  // le code ci-dessous ...
  Method* pM = pEv->GetMethod(DEFINE_NEXTEVENTCLOCKTIME);
  pM->SetArgValue(PREVIOUS_EVENT_ARGUMENT, pEv);
  nextEventClock = ((EventIntMethod*)pM)->Body((EventMethod*)pM);

  // ... est équivalent à celui-ci
  pEv->SetEventArgValue(DEFINE_NEXTEVENTCLOCKTIME, PREVIOUS_EVENT_ARGUMENT, pEv);
  nextEventClock = pEv->ExecIntMethod(DEFINE_NEXTEVENTCLOCKTIME);
  ...
}[/P]
\end{verbatim}
 */
//@{
         ///
class DefineNextEventClockTime : public IntEventMethod
{
public:
   DefineNextEventClockTime();           
 }; // _end class_

//@} // fin classe

// X///////////////X/////////////////////////////X////////////////////////////

/**@name Calcul de la date aléatoire de cet événement

Le {\em corps} de cette méthode peut être surchargé par l'utilisateur, avec [Y]SetRandomizeThisEventClockTime[/Y]. Il établit et renvoie la date de cet événement, en exploitant les deux arguments prédéfinis dont les valeurs sont la borne inférieure et la borne supérieure de la date. L'usage de cette méthode n'a bien sûr d'intérêt que lorsque les deux bornes sont différentes (utiliser [Y]SetOccurrenceClockTime(int)[/Y] sinon).\index{RandomizeThisEventClockTime} 
\begin{verbatim}
[P]int myEvent_samplingThisEventClockTime_Body(EventMethod* pM) {
  int lb = pM->GetIntArgValue(INT_LOW_BOUND_ARGUMENT);
  int ub = pM->GetIntArgValue(INT_UPPER_BOUND_ARGUMENT);
  int thisEventClock;
  ... // détermination aléatoire de la date
  return thisEventClock;
}
MyEvent::MyEvent() 
   : Event(MY_EVENT, "myEvent") {
  ...
  SetRandomizeThisEventClockTime(myEvent_samplingThisEventClockTime_Body);
  ...
};[/P]
\end{verbatim}

On rappelle (voir page 'La classe des événements'), que lorsque l'utilisateur n'a pas surchargé ce corps, le moteur de simulation met en oeuvre un tirage uniforme entre borne inférieure et borne supérieure.

Le constructeur [Y]RandomizeThisEventClockTime()[/Y] affecte le symbole de classe RANDOMIZE_THISEVENTCLOCKTIME et le nom de classe "randomizeThisEventClockTime". Ainsi, on peut écrire :
\begin{verbatim}
[P]int main(...) {
  ...
  Event* pEv = new MyEvent();
  int thisEventClock;
  // le code ci-dessous ...
  Method* pM = pEv->GetMethod(RANDOMIZE_THISEVENTCLOCKTIME);
  pM->SetArgValue(INT_LOW_BOUND_ARGUMENT, 0);
  pM->SetArgValue(INT_UPPER_BOUND_ARGUMENT, 9);
  thisEventClock = ((EventIntMethod*)pM)->Body((EventMethod*)pM);

  // ... est équivalent à celui-ci
  pEv->SetIntArgValue(RANDOMIZE_THISEVENTCLOCKTIME, INT_LOW_BOUND_ARGUMENT, 0);
  pEv->SetIntArgValue(RANDOMIZE_THISEVENTCLOCKTIME, INT_UPPER_BOUND_ARGUMENT, 9);
  thisEventClock = pEv->ExecIntMethod(RANDOMIZE_THISEVENTCLOCKTIME);
  ...
  SetOccurrenceClockTime(thisEventClock);
}[/P]
\end{verbatim}
 */
//@{
         ///
class RandomizeThisEventClockTime : public IntEventMethod
{
public:
   RandomizeThisEventClockTime();           
 };   // _end class_

//@} // fin classe

       

// X///////////////X/////////////////////////////X////////////////////////////

/**@name Calcul de la date aléatoire de l'événement généré automatiquement
Cette méthode n'est pas utilisée par le moteur de simulation, mais seulement, et éventuellement, dans le code de l'application, plus précisément dans l'une ou l'autre des méthodes d'autogénération d'un événement. Le corps, affecté par [Y]SetRandomizeNextEventClockTime[/Y], établit et renvoie la date d'un événement autogénéré, en exploitant les deux arguments prédéfinis dont les valeurs sont la borne inférieure et la borne supérieure de la date.\index{RandomizeNextEventClockTime}
\begin{verbatim}
[P]int myEvent_samplingNextEventClockTime_Body(EventMethod* pM) {
  int lb = pM->GetIntArgValue(INT_LOW_BOUND_ARGUMENT);
  int ub = pM->GetIntArgValue(INT_UPPER_BOUND_ARGUMENT);
  int nextEventClock;
  ... // détermination aléatoire de la date
  return nextEventClock;
}
MyEvent::MyEvent() 
   : Event(MY_EVENT, "myEvent") {
  ...
  SetRandomizeNextEventClockTime(myEvent_samplingNextEventClockTime_Body);
  ...
};[/P]
\end{verbatim}

On rappelle (voir page 'La classe des événements'), que lorsque l'utilisateur n'a pas surchargé ce corps, ou que l'utilisateur n'utilise pas cette méthode dans son code, le moteur de simulation met en oeuvre un tirage uniforme entre borne inférieure et borne supérieure quand elles sont spécifiées. Quand elles ne sont pas spécifiées, l'utilisateur utilise alors le service [Y]SetOccurrenceClockTime(int)[/Y].

Le constructeur [Y]RandomizeNextEventClockTime()[/Y] affecte le symbole de classe RANDOMIZE_NEXTEVENTCLOCKTIME et le nom de classe "randomizeNextEventClockTime". Ainsi, on peut écrire :
\begin{verbatim}
[P]bool pingEvent_generateNextEvent_Body(EventMethod* pM) {
  bool succeed = TRUE;
  Event* pCurrentEvent = pM->DescribedEvent();
  Event* pNextEvent = new PongEvent();
  int nextEventClock;
  // le code ci-dessous ...
  Method* pM = pCurrentEvent->GetMethod(RANDOMIZE_NEXTEVENTCLOCKTIME);
  pM->SetArgValue(INT_LOW_BOUND_ARGUMENT, 0);
  pM->SetArgValue(INT_UPPER_BOUND_ARGUMENT, 9);
  nextEventClock = ((EventIntMethod*)pM)->Body((EventMethod*)pM);

  // ... est équivalent à celui-ci
  pEv->SetIntArgValue(RANDOMIZE_NEXTEVENTCLOCKTIME, INT_LOW_BOUND_ARGUMENT, 0);
  pEv->SetIntArgValue(RANDOMIZE_NEXTEVENTCLOCKTIME, INT_UPPER_BOUND_ARGUMENT, 9);
  thisEventClock = pEv->ExecIntMethod(RANDOMIZE_NEXTEVENTCLOCKTIME);
  ...
  pNextEvent->SetOccurrenceClockTime(nextEventClock);
  ...
}[/P]
\end{verbatim}
 */
//@{
         ///
class RandomizeNextEventClockTime : public IntEventMethod
{
public:
   RandomizeNextEventClockTime();           
 }; // _end class_

//@} // fin classe 

// X///////////////X/////////////////////////////X////////////////////////////

/* *@name Détermine si la boucle des dépilements doit être arrêtée quand cet événement est en tête de l'agenda.\\
Le corps, affecté par [Y]SetIsPoppingToSuspend[/Y], doit renvoyer 'vrai' si et seulement si la boucle de dépilement des événements de l'agenda doit être arrêtée quand l'événement sur lequel porte cette méthode est en tête de l'agenda.
\index{IsPoppingToSuspend} 
\newline Le constructeur [Y]IsPoppingToSuspend()[/Y] affecte le symbole de classe IS_POPPING_TO_SUSPEND et le nom de classe "isPoppingToSuspend".
 */
//@{
         // /
class IsPoppingToSuspend : public EventPredicate
{
public:
   IsPoppingToSuspend();           
 }; // _end class_

//@} // fin classe


//@}  // fin méthodes événements






// X///////////////X/////////////////////////////X////////////////////////////

/**@name Méthodes prédéfinies de démons 

De manière générale, le corps d'une de ces méthodes spécifie la réaction ... :\index{WhenSetInt}
\begin{itemize}
\item à l'affectation d'une valeur à un descripteur variable 
\item à l'accès à une valeur d'un descripteur variable 
\item à l'ajout ou au retrait d'un élément d'une entité
\end{itemize}

On suit ci-dessous l'exemple de l'affectation d'une valeur entière. C'est l'utilisateur qui code ce {\em corps} et qui l'affecte à la méthode (ici avec [Y]AssignWhenSetIntMethod[/Y]), de la manière suivante :\\

\begin{verbatim}
[P]IntDescValueMonitor* SizeMonitor;

int Size_whenSet(MonitorMethod* pM) {
  Descriptor* pVD = pM->GetDescriptorArgValue(MONITOREDDESCRIPTOR_ARGUMENT);
  //int oldValue = pM->GetIntArgValue(CURRENTINTVALUE_ARGUMENT);
  int candidateValue = pM->GetIntArgValue(CANDIDATEINTVALUE_ARGUMENT);
  ... // la réaction
  return candidateValue;
}
void App_DefineMonitors() {
  SizeMonitor = new IntDescValueMonitor("sizeMonitor");
  SizeMonitor->AssignWhenSetIntMethod(Size_whenSet);
  ...
};[/P]
\end{verbatim}

Par contre, l'utilisateur n'invoque pas lui-même le corps de ce type de méthode. C'est le moteur qui le fait, lorsqu'il reçoit une instruction dans la liste du haut de la page, et si un démon est prévu pour cet événement. On indique cepandant ici la manière dont le moteur réalise cet appel (en connaissance que le constructeur [Y]WhenSetInt()[/Y] affecte le symbole de classe WHEN_SET_INT et le nom de classe "whenSetInt") :
\begin{verbatim}
[P]void  IntVariableDescriptor::SetValue(int value) {
  ...
  IntDescValueMonitor* pDVM = ...;// accès à une donnée membre privée du descripteuur
  Method* pM = pDVM->GetMethod(WHEN_SET_INT);
  pM->SetArgValue(MONITOREDDESCRIPTOR_ARGUMENT, this);
  pM->SetArgValue(CURRENTINTVALUE_ARGUMENT, mValue);  // mValue est une donnée membre privée
  pM->SetArgValue(CANDIDATEINTVALUE_ARGUMENT, value);
  mValue = ((IntMonitorMethod*)pM)->Body((MonitorMethod*)pM);
  ...
}[/P]
\end{verbatim}

{\large {\tt Methodes de démons sur la valeur des descripteurs}}\\

Sur ce modèle, les autres méthodes prédéfinies des démons sur la valeur d'un descripteur ([C]DescValueMonitor[/C]) sont les suivantes, avec leurs arguments spécifiques. Il y a en effet un argument commun à toutes : l'argument [C]MonitoredDescriptorArgument[/C] de symbole MONITOREDDESCRIPTOR_ARGUMENT et de nom "monitoredDescriptorArgument", dont l'usage est montré dans le code ci-dessus. Sa valeur est le descripteur sur lequel le démon surveille un événement (affectation/accès).\\

L'argument de symbole CURRENT[...]VALUE_ARGUMENT a toujours pour valeur (affectée par le moteur de simulation) la valeur du descripteur juste avant la tentative d'affectation d'une nouvelle valeur, ou juste avant l'accès à la valeur.\\
L'argument de symbole CANDIDATE[...]VALUE_ARGUMENT a toujours pour valeur (affectée par le moteur de simulation) la nouvelle valeur qu'on tente d'affecter au descripteur.\\
L'argument de symbole CURRENTENTITYTABVALUE_ARGUMENT a pour valeur le tableau d'entités juste avant la tentative d'ajout ou de retrait d'un élément.\\
L'argument de symbole CANDIDATEENTITYVALUE_ARGUMENT, pour la méthode [C]WhenAddEntity[/C] (resp. [C]WhenRemoveEntity[/C]) a pour valeur l'entités qu'on tente d'ajouter (resp. de retirer).\\
L'argument de symbole CURRENTDESCVALUESPECTABVALUE_ARGUMENT a pour valeur le tableau de spécifications de domaine de valeurs juste avant la tentative d'ajout ou de retrait d'un élément.\\
L'argument de symbole CANDIDATEDESCVALUESPECVALUE_ARGUMENT, pour la méthode [C]WhenAddDescValueSpec[/C] (resp. [C]WhenRemoveDescValueSpec[/C]) a pour valeur la spécification de domaine de valeurs qu'on tente d'ajouter (resp. de retirer).\\

\begin{tabular}{cccc}
{\tiny classe du démon} & {\tiny classe de la méthode} & {\tiny type retourné par le corps} & {\tiny symbole de classe des arguments du corps}\\ \hline
\hline
{\tiny IntDescValueMonitor} & {\tiny WhenSetInt} & {\tiny int} & {\tiny CURRENTINTVALUE_ARGUMENT}\\
            &               &                                  & {\tiny CANDIDATEINTVALUE_ARGUMENT}\\ \hline
{\tiny FloatDescValueMonitor} & {\tiny WhenSetFloat} & {\tiny float} & {\tiny CURRENTFLOATVALUE_ARGUMENT}\\
            &               &                                  & {\tiny CANDIDATEFLOATVALUE_ARGUMENT}\\ \hline
{\tiny DoubleDescValueMonitor} & {\tiny WhenSetDouble} & {\tiny double} & {\tiny CURRENTDOUBLEVALUE_ARGUMENT}\\
            &               &                                  & {\tiny CANDIDATEDOUBLEVALUE_ARGUMENT}\\ \hline
{\tiny StringDescValueMonitor} & {\tiny WhenSetString} & {\tiny char*} & {\tiny CURRENTSTRINGVALUE_ARGUMENT}\\
            &               &                                  & {\tiny CANDIDATESTRINGVALUE_ARGUMENT}\\ \hline
{\tiny EntityDescValueMonitor} & {\tiny WhenSetEntity} & {\tiny BasicEntity*} & {\tiny CURRENTENTITYVALUE_ARGUMENT}\\
            &               &                                  & {\tiny CANDIDATEENTITYVALUE_ARGUMENT}\\ \hline
{\tiny EntityTabDescValueMonitor} & {\tiny WhenSetEntityTab} & {\tiny pEntityTab*} & {\tiny CURRENTENTITYTABVALUE_ARGUMENT}\\
            &               &                                  & {\tiny CANDIDATEENTITYTABVALUE_ARGUMENT}\\ \hline
{\tiny EntityTabDescValueMonitor} & {\tiny WhenAddEntity} & {\tiny BasicEntity*} & {\tiny CURRENTENTITYTABVALUE_ARGUMENT}\\
            &               &                                  & {\tiny CANDIDATEENTITYVALUE_ARGUMENT}\\ \hline
{\tiny EntityTabDescValueMonitor} & {\tiny WhenRemoveEntity} & {\tiny BasicEntity*} & {\tiny CURRENTENTITYTABVALUE_ARGUMENT}\\
            &               &                                  & {\tiny CANDIDATEENTITYVALUE_ARGUMENT}\\ \hline
{\tiny DescValueSpeTabDescValueMonitor} & {\tiny WhenAddDescValueSpec} & {\tiny DescValueSpec*} & {\tiny CURRENTDESCVALUESPECTABVALUE_ARGUMENT}\\
            &               &                                  & {\tiny CANDIDATEDESCVALUESPECVALUE_ARGUMENT}\\ \hline
{\tiny DescValueSpeTabDescValueMonitor} & {\tiny WhenRemoveDescValueSpec} & {\tiny DescValueSpec*} & {\tiny CURRENTDESCVALUESPECTABVALUE_ARGUMENT}\\
            &               &                                  & {\tiny CANDIDATEDESCVALUESPECTABVALUE_ARGUMENT}\\ \hline
{\tiny IntDescValueMonitor} & {\tiny WhenGetInt} & {\tiny int} & {\tiny CURRENTINTVALUE_ARGUMENT}\\ \hline
{\tiny FloatDescValueMonitor} & {\tiny WhenGetFloat} & {\tiny float} & {\tiny CURRENTFLOATVALUE_ARGUMENT}\\ \hline
{\tiny DoubleDescValueMonitor} & {\tiny WhenGetDouble} & {\tiny double} & {\tiny CURRENTDOUBLEVALUE_ARGUMENT}\\ \hline
{\tiny StringDescValueMonitor} & {\tiny WhenGetString} & {\tiny char*} & {\tiny CURRENTSTRINGVALUE_ARGUMENT}\\ \hline
{\tiny EntityDescValueMonitor} & {\tiny WhenGetEntity} & {\tiny BasicEntity*} & {\tiny CURRENTENTITYVALUE_ARGUMENT}\\ \hline
{\tiny EntityTabDescValueMonitor} & {\tiny WhenGetEntityTab} & {\tiny pEntityTab*} & {\tiny CURRENTENTITYTABVALUE_ARGUMENT}\\ \hline
{\tiny DescValueSpeTabDescValueMonitor} & {\tiny WhenGetDescValueSpecTab} & {\tiny pDescValueSpecTab*} & {\tiny CURRENTDESCVALUESPECTABVALUE_ARGUMENT}\\ \hline
\end{tabular}


{\large {\tt Methodes de démons sur la structure des entités}}\\

Les méthodes prédéfinies des démons sur la structure d'une entité ([C]StructureMonitor[/C]) sont les suivantes. Les deux méthodes ont deux arguments : \\
- l'argument [C]CurrentEntityContainerArgument[/C] de symbole CURRENTENTITYCONTAINER_ARGUMENT et de nom "currentEntityContainerArgument". Sa valeur est l'entité de laquelle un tente d'ajouter ou d'enlever un élément.\\
- l'argument [C]CandidateEntityElementArgument[/C] de symbole CANDIDATEENTITYELEMENT_ARGUMENT et de nom "candidateEntityElementArgument". Sa valeur est l'élément qu'on tente d'ajouter ou d'enlever de l'entité.\\

\begin{tabular}{cccc}
{\tiny classe du démon} & {\tiny classe de la méthode} & {\tiny type retourné par le corps} & {\tiny symbole de classe des arguments du corps}\\ \hline
\hline
{\tiny StructureMonitor} & {\tiny WhenAddElement} & {\tiny BasicEntity*} & \\
            &               &                                  & \\ \hline
{\tiny StructureMonitor} & {\tiny WhenRemoveElement} & {\tiny BasicEntity*} &\\
            &               &                                  & \\ \hline
\end{tabular}
*/ 
//@{

// X///////////////X/////////////////////////////X////////////////////////////

/* *@name WhenSetInt
Le corps, affecté par [Y]AssignWhenSetIntMethod[/Y], établit la réaction à l'affectation d'une valeur à un descripteur variable à valeur entière.\index{WhenSetInt} \newline Le constructeur [Y]WhenSetInt()[/Y] affecte le symbole de classe WHEN_SET_INT et le nom de classe "whenSetInt".
 */
//@{
        // /
class WhenSetInt : public IntMonitorMethod
{
public:
   WhenSetInt();           
 }; // _end class_

//@} // fin classe

/* *@name WhenSetFloat
Le corps, affecté par [Y]AssignWhenSetFloatMethod[/Y], établit la réaction à l'affectation d'une valeur à un descripteur variable à valeur flottante.\index{WhenSetFloat} \newline Le constructeur [Y]WhenSetFloat()[/Y] affecte le symbole de classe WHEN_SET_FLOAT et le nom de classe "whenSetFloat".
 */
//@{
        // /
class WhenSetFloat : public FloatMonitorMethod
{
public:
   WhenSetFloat();           
 }; // _end class_

//@} // fin classe

/* *@name WhenSetDouble
Le corps, affecté par [Y]AssignWhenSetDoubleMethod[/Y], établit la réaction à l'affectation d'une valeur à un descripteur variable à valeur flottante double précision.\index{WhenSetDouble} \newline Le constructeur [Y]WhenSetDouble()[/Y] affecte le symbole de classe WHEN_SET_DOUBLE et le nom de classe "whenSetDouble".
 */
//@{
        // /
class WhenSetDouble : public DoubleMonitorMethod
{
public:
   WhenSetDouble();           
 }; // _end class_

//@} // fin classe

/* *@name WhenSetString
Le corps, affecté par [Y]AssignWhenSetStringMethod[/Y], établit la réaction à l'affectation d'une valeur à un descripteur variable à valeur chaîne de caractères.\index{WhenSetString} \newline Le constructeur [Y]WhenSetString()[/Y] affecte le symbole de classe WHEN_SET_STRING et le nom de classe "whenSetString".
 */
//@{
        // /
class WhenSetString : public StringMonitorMethod
{
public:
   WhenSetString();           
 }; // _end class_

//@} // fin classe

/* *@name WhenSetEntity
Le corps, affecté par [Y]AssignWhenSetEntityMethod[/Y], établit la réaction à l'affectation d'une valeur à un descripteur variable dont la valeur est un pointeur sur une [C]Entity[/C].\index{WhenSetEntity} \newline Le constructeur [Y]WhenSetEntity()[/Y] affecte le symbole de classe WHEN_SET_ENTITY et le nom de classe "whenSetEntity".
 */
//@{
        // /
class WhenSetEntity : public EntityMonitorMethod
{
public:
   WhenSetEntity();           
 }; // _end class_

//@} // fin classe

/* *@name WhenSetEntityTab
Le corps, affecté par [Y]AssignWhenSetEntityTabMethod[/Y], établit la réaction à l'affectation d'une valeur à un descripteur variable dont la valeur est un tableau de pointeurs sur des [C]Entity[/C].\index{WhenSetEntityTab} \newline Le constructeur [Y]WhenSetEntityTab()[/Y] affecte le symbole de classe WHEN_SET_ENTITYTAB et le nom de classe "whenSetEntityTab".
 */
//@{
        // /
class WhenSetEntityTab : public EntityTabMonitorMethod
{
public:
   WhenSetEntityTab();           
 }; // _end class_

//@} // fin classe

/* *@name WhenSetDescValueSpecTab
Le corps, affecté par [Y]AssignWhenSetDescValueSpecTabMethod[/Y], établit la réaction à l'affectation d'une valeur à un descripteur variable dont la valeur est un tableau de pointeurs sur des [C]DescValueSpec[/C].\index{WhenSetDescValueSpecTab} \newline Le constructeur [Y]WhenSetDescValueSpecTab()[/Y] affecte le symbole de classe WHEN_SET_DESCVALUESPECTAB et le nom de classe "whenSetDescValueSpecTab".
 */
//@{
        // /
class WhenSetDescValueSpecTab : public DescValueSpecTabMonitorMethod
{
public:
   WhenSetDescValueSpecTab();           
 }; // _end class_

//@} // fin classe

/* *@name WhenAddEntity
Le corps, affecté par [Y]AssignWhenAddEntityMethod[/Y], établit la réaction à l'ajout d'un élément de valeur à un descripteur variable dont la valeur est un tableau de pointeurs sur des [C]Entity[/C].\index{WhenAddEntity} \newline Le constructeur [Y]WhenAddEntity()[/Y] affecte le symbole de classe WHEN_ADD_ENTITY et le nom de classe "whenAddEntity".
 */
//@{
        // /
class WhenAddEntity : public EntityMonitorMethod
{
public:
   WhenAddEntity();           
 }; // _end class_

//@} // fin classe

/* *@name WhenRemoveEntity
Le corps, affecté par [Y]AssignWhenRemoveEntityMethod[/Y], établit la réaction à l'ajout d'un élément de valeur à un descripteur variable dont la valeur est un tableau de pointeurs sur des [C]Entity[/C].\index{WhenRemoveEntity} \newline Le constructeur [Y]WhenRemoveEntity()[/Y] affecte le symbole de classe WHEN_REMOVE_ENTITY et le nom de classe "whenRemoveEntity".
 */
//@{
        // /
class WhenRemoveEntity : public EntityMonitorMethod
{
public:
   WhenRemoveEntity();            
 }; // _end class_

//@} // fin classe

/* *@name WhenAddDescValueSpec
Le corps, affecté par [Y]AssignWhenAddDescValueSpecMethod[/Y], établit la réaction à l'ajout d'un élément de valeur à un descripteur variable dont la valeur est un tableau de pointeurs sur des [C]DescValueSpec[/C].\index{WhenAddDescValueSpec} \newline Le constructeur [Y]WhenAddDescValueSpec()[/Y] affecte le symbole de classe WHEN_ADD_DESCVALUESPEC et le nom de classe "whenAddDescValueSpec".
 */
//@{
        // /
class WhenAddDescValueSpec : public DescValueSpecMonitorMethod
{
public:
   WhenAddDescValueSpec();           
 }; // _end class_

//@} // fin classe

/* *@name WhenRemoveDescValueSpec
Le corps, affecté par [Y]AssignWhenRemoveDescValueSpecMethod[/Y], établit la réaction à l'ajout d'un élément de valeur à un descripteur variable dont la valeur est un tableau de pointeurs sur des [C]DescValueSpec[/C].\index{WhenRemoveDescValueSpec} \newline Le constructeur [Y]WhenRemoveDescValueSpec()[/Y] affecte le symbole de classe WHEN_REMOVE_DESCVALUESPEC et le nom de classe "whenRemoveDescValueSpec".
 */
//@{
        // /
class WhenRemoveDescValueSpec : public DescValueSpecMonitorMethod
{
public:
   WhenRemoveDescValueSpec();            
 };  // _end class_

//@} // fin classe

/* *@name WhenGetInt
Le corps, affecté par [Y]AssignWhenGetIntMethod[/Y], établit la réaction à l'accès à la valeur d'un descripteur variable à valeur entière.\index{WhenGetInt} \newline Le constructeur [Y]WhenGetInt()[/Y] affecte le symbole de classe WHEN_GET_INT et le nom de classe "whenGetInt".
 */
//@{
        // /
class WhenGetInt : public IntMonitorMethod
{
public:
   WhenGetInt();           
 }; // _end class_

//@} // fin classe

/* *@name WhenGetFloat
Le corps, affecté par [Y]AssignWhenGetFloatMethod[/Y], établit la réaction à l'accès à la valeur d'un descripteur variable à valeur flottante.\index{WhenGetFloat} \newline Le constructeur [Y]WhenGetFloat()[/Y] affecte le symbole de classe WHEN_GET_FLOAT et le nom de classe "whenGetFloat".
 */
//@{
        // /
class WhenGetFloat : public FloatMonitorMethod
{
public:
   WhenGetFloat();           
 }; // _end class_

//@} // fin classe

/* *@name WhenGetDouble
Le corps, affecté par [Y]AssignWhenGetDoubleMethod[/Y], établit la réaction à l'accès à la valeur d'un descripteur variable à valeur flottante double précision.\index{WhenGetDouble} \newline Le constructeur [Y]WhenGetDouble()[/Y] affecte le symbole de classe WHEN_GET_DOUBLE et le nom de classe "whenGetDouble".
 */
//@{
        // /
class WhenGetDouble : public DoubleMonitorMethod
{
public:
   WhenGetDouble();           
 }; // _end class_

//@} // fin classe

/* *@name WhenGetString
Le corps, affecté par [Y]AssignWhenGetStringMethod[/Y], établit la réaction à l'accès à la valeur d'un descripteur variable à valeur chaîne de caractères.\index{WhenGetString} \newline Le constructeur [Y]WhenGetString()[/Y] affecte le symbole de classe WHEN_GET_STRING et le nom de classe "whenGetString".
 */
//@{
        // /
class WhenGetString : public StringMonitorMethod
{
public:
   WhenGetString();           
 }; // _end class_

//@} // fin classe

/* *@name WhenGetEntity
Le corps, affecté par [Y]AssignWhenGetEntityMethod[/Y], établit la réaction à l'accès à la valeur d'un descripteur variable dont la valeur est un pointeur sur une [C]Entity[/C].\index{WhenGetEntity} \newline Le constructeur [Y]WhenGetEntity()[/Y] affecte le symbole de classe WHEN_GET_ENTITY et le nom de classe "whenGetEntity".
 */
//@{
        // /
class WhenGetEntity : public EntityMonitorMethod
{
public:
   WhenGetEntity();           
 };  // _end class_

//@} // fin classe

/* *@name WhenGetEntityTab
Le corps, affecté par [Y]AssignWhenGetEntityTabMethod[/Y], établit la réaction à l'accès à la valeur d'un descripteur variable dont la valeur est un tableau de pointeurs sur des [C]Entity[/C].\index{WhenGetEntityTab} \newline Le constructeur [Y]WhenGetEntityTab()[/Y] affecte le symbole de classe WHEN_GET_ENTITYTAB et le nom de classe "whenGetEntityTab".
 */
//@{
        // /
class WhenGetEntityTab : public EntityTabMonitorMethod
{
public:
   WhenGetEntityTab();           
 };  // _end class_

//@} // fin classe

/* *@name WhenGetDescValueSpecTab
Le corps, affecté par [Y]AssignWhenGetDescValueSpecTabMethod[/Y], établit la réaction à l'accès à la valeur d'un descripteur variable dont la valeur est un tableau de pointeurs sur des [C]DescValueSpec[/C].\index{WhenGetDescValueSpecTab} \newline Le constructeur [Y]WhenGetDescValueSpecTab()[/Y] affecte le symbole de classe WHEN_GET_DESCVALUESPECTAB et le nom de classe "whenGetDescValueSpecTab".
 */
//@{
        // /
class WhenGetDescValueSpecTab : public DescValueSpecTabMonitorMethod
{
public:
   WhenGetDescValueSpecTab();           
 };  // _end class_

//@} // fin classe 

/* *@name WhenAddElement
Le corps, affecté par [Y]AssignWhenAddElementMethod[/Y], établit la réaction à l'ajout d'un élément à une instance d'[C]Entity[/C].\index{WhenAddElement} \newline Le constructeur [Y]WhenAddElement()[/Y] affecte le symbole de classe WHEN_ADD_ELEMENT et le nom de classe "whenAddElement".
 */
//@{
        // /
class WhenAddElement : public EntityMonitorMethod
{
public:
   WhenAddElement();           
 }; // _end class_

//@} // fin classe

/* *@name WhenRemoveElement
Le corps, affecté par [Y]AssignWhenRemoveElementMethod[/Y], établit la réaction au retrait d'un élément à une instance d'[C]Entity[/C].\index{WhenRemoveElement} \newline Le constructeur [Y]WhenRemoveElement()[/Y] affecte le symbole de classe WHEN_REMOVE_ELEMENT et le nom de classe "whenRemoveElement".
 */
//@{
        // /
class WhenRemoveElement : public VoidMonitorMethod
{
public:
   WhenRemoveElement(); 
 };  // _end class_

//@} // fin classe


//@}  // fin méthodes démons


// X///////////////X/////////////////////////////X////////////////////////////

/**@name Méthodes prédéfinies de spécifications d'ensembles d'entités  

De manière générale, le corps d'une de ces méthodes spécifie un ensemble d'entités ... :\index{WhenSetInt}
\begin{itemize}
\item qu'on va sélectionner dans un ensemble préétabli
\item qu'on va créer par instantiation (en nombre préétabli ou non)
\end{itemize}

{\large {\tt Lorsqu'il s'agit de sélectionner}}, l'utilisateur code une fonction comme suit :\\

\begin{verbatim}
[P]bool myEntitySpec_selectorPredicate_Body(EntitySpecMethod* pM) {
  bool result = false;
  Entity* pCandidate = pM->GetEntityArgValue(CANDIDATEENTITY_ARGUMENT);
  Entity* pEref = pM->GetEntityArgValue(REFERENCEENTITY_ARGUMENT);
  // détermination si pCandidate doit être sélectionnée ou non
  return result;
};[/P]
\end{verbatim}

Puis, par [Y]AssignSelectorPredicate[/Y], il l'attache comme corps de la méthode [C]EntitySelector[/C] d'une sous-classe d'[C]EntitySpec[/C] de son domaine d'application :\\

\begin{verbatim}
[P]MyEntitySpec::MyEntitySpec() : EntitySpec(SOME_ENTITY_CLASS_ID) {
	ClassName("myEntitySpec");
	SetQuantificatorId(NFIRST);
	SetMode(RETURN_INSTANCES);
	IntQuantity(1);
	AssignSelectorPredicate(myEntitySpec_selectorPredicate_Body);
};[/P]
\end{verbatim}


{\large {\tt Lorsqu'il s'agit d'instancier}} des entités que le moteur de simulation va assembler lui-même dans une liste, l'utilisateur code une fonction comme suit :\\

\begin{verbatim}
[P]BasicEntity* myEntitySpec_entityInstantiator_Body(EntitySpecMethod* pM) {
  Entity* pEref = pM->GetEntityArgValue(REFERENCEENTITY_ARGUMENT);
  UserClassId classId = pM->GetIntArgValue(CLASSID_ARGUMENT);
  Entity* result = App_NewEntityFromId(classId);
  ... // compléments de construction de l'entité qui sera renvoyée
  return result;
};[/P]
\end{verbatim}

L'attachage comme corps de la méthode [C]EntityInstantiator[/C] d'une sous-classe d'[C]EntitySpec[/C] du domaine d'application se fait par [Y]AssignEntityInstantiator[/Y].\\


{\large {\tt Lorsqu'il s'agit d'instancier}} des entités et de les assembler dans une liste, l'utilisateur code une fonction comme suit :\\

\begin{verbatim}
[P]pEntityTab* myEntitySpec_entityListInstantiator_Body(EntitySpecMethod* pM) {
  Entity* pEref = pM->GetEntityArgValue(REFERENCEENTITY_ARGUMENT);
  UserClassId classId = pM->GetIntArgValue(CLASSID_ARGUMENT);
  int listSise = pM->GetIntArgValue(LISTSIZE_ARGUMENT); 
  pEntityTab* result = new pEntityTab();
  for(int k=0;k<listSize;k++) {
    Entity* pE_k = App_NewEntityFromId(classId);
    ... // compléments de construction de l'entité qui sera renvoyée
	  result->push_back(pE_k);
  }
  return result;
};[/P]
\end{verbatim}

L'attachage comme corps de la méthode [C]EntityListInstantiator[/C] d'une sous-classe d'[C]EntitySpec[/C] du domaine d'application se fait par [Y]AssignEntityListInstantiator[/Y].\\

{\large {\tt Pour la spécification du nombre d'entités}} à sélectionner ou à instancier, l'utilisateur code une fonction comme suit :\\

\begin{verbatim}
[P]int myEntitySpec_quantityEvaluator_Body(EntitySpecMethod* pM) {
  Entity* pEref = pM->GetEntityArgValue(REFERENCEENTITY_ARGUMENT);
  int result;
  ... // détermination du nombre
  return result;
};[/P]
\end{verbatim}

L'attachage comme corps de la méthode [C]QuantityEvaluator[/C] d'une sous-classe d'[C]EntitySpec[/C] du domaine d'application se fait par [Y]AssignQuantityEvaluator[/Y].
*/ 
//@{

// X///////////////X/////////////////////////////X////////////////////////////

/**@name EntitySelector
Le corps, affecté par [Y]AssignSelectorPredicate[/Y], détermine si l'entité passée en argument doit être sélectionnée. Une entité pouvant servir de référence est passée en argument.\index{EntitySelector} \newline Le constructeur [Y]EntitySelector()[/Y] affecte le symbole de classe SELECTOR_PREDICATE et le nom de classe "selectorPredicate". 
 */
//@{
        ///
class EntitySelector : public EntitySpecPredicate
{
public:
   EntitySelector();           
}; // _end class_

//@} // fin classe

// X///////////////X/////////////////////////////X////////////////////////////

/**@name EntityInstantiator
Le corps, affecté par [Y]AssignEntityInstantiator[/Y], renvoie une instance nouvellement créée de la classe dont le symbole est passé en argument. Une entité pouvant servir de référence est passée en argument.\index{EntityInstantiator}. \newline Le constructeur [Y]EntityInstantiator()[/Y] affecte le symbole de classe ENTITY_INSTANTIATOR et le nom de classe "entityInstantiator". 
 */
//@{
        ///
class EntityInstantiator : public EntityEntitySpecMethod
{
public:
   EntityInstantiator();           
 }; // _end class_

//@} // fin classe

// X///////////////X/////////////////////////////X////////////////////////////

/**@name EntityListInstantiator
Le corps, affecté par [Y]AssignEntityInstantiator[/Y], renvoie une liste d'instances nouvellement créées de la classe dont le symbole est passé en argument. La taille de la liste est aussi passée en argument, de même qu'une entité pouvant servir de référence.\index{EntityListInstantiator} \newline Le constructeur [Y]EntityListInstantiator()[/Y] affecte le symbole de classe ENTITYLIST_INSTANTIATOR et le nom de classe "entityListInstantiator".
 */
//@{
        ///
class EntityListInstantiator : public EntityTabEntitySpecMethod
{
public:
   EntityListInstantiator();           
};  // _end class_

//@} // fin classe

// X///////////////X/////////////////////////////X////////////////////////////

/**@name QuantityEvaluator 
Le corps, affecté par [Y]AssignQuantityEvaluator[/Y], détermine la taille de la liste d'entités à renvoyer.\index{QuantityEvaluator} \newline Le constructeur [Y]QuantityEvaluator()[/Y] affecte le symbole de classe QUANTITY_EVALUATOR et le nom de classe "quantityEvaluator". 
 */
//@{
        ///
class QuantityEvaluator : public IntEntitySpecMethod
{
public:
   QuantityEvaluator();           
};  // _end class_

//@} // fin classe

//@}  // fin méthodes d'EntitySpec

// X///////////////X/////////////////////////////X////////////////////////////

/* *@name Méthodes prédéfinies de spécifications de domaines de valeur */ 
// @{

// X///////////////X/////////////////////////////X////////////////////////////

/* *@name SpecSatisfied  (- JPR 110623 unused)
Le corps détermine si l'entité passée en argument satisfait l'instance de [C]DescValueSpec[/C], autrement dit si la valeur de son descripteur dont le symbole de classe est la valeur du descripteur DESCRIPTOR_ID est bien dans le domaine définie par la spécifications de domaine de valeur.
 */
// @{
        //
/*
class SpecSatisfied : public EntityPredicate
{
public:
   SpecSatisfied();           
};  // _end class_
*/

// @} // fin classe

// @}  // fin méthodes de DescValueSpec

// X///////////////X/////////////////////////////X////////////////////////////

//@}  // fin méthodes prédéfinies

//@}  // fin chapitre méthodes

#endif  // COREMETHOD_HH
