//============================================================================
// FILE: CoreDiscreteProcess.h
//
// TYPE: C++CLASS
// DESCRIPTION: entête des divers descripteurs constants de Diese
// 
// AUTHORS: Rellier (INRA/UBIA, Toulouse)
//=X===============X=============================X============================

#ifndef _COREDISCRETEPROCESS_HH
#define _COREDISCRETEPROCESS_HH

// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//
#include "DiscreteProcess.h"
#include "File.h"

/**@name CDPFORTEXLes processus discrets de lecture d'un fichier de données factuelles

Deux sous-classes de processus discrets permettent d'installer un mécanisme de lecture, enregistrement par enregistrement, d'un fichier de données factuelles. On rappelle seulement ses enregistrements correspondent chacun à un instant quelconque, précisé dans les champs, et qu'ils sont rangés dans le fichier dans l'ordre croissant des instants.\\

Ce mécanisme nécessite la déclaration préalable d'une classe de [C]FactualDataFile[/C] : on ne pourra lire que des fichiers physiques qui auront fait l'objet d'une instantiation de cette classe (fichier logique). Le symbole de la sous-classe en question de [C]FactualDataFile[/C] est une propriété des processus (accessible par [Y]FileClassId()[/Y]).\\

Les deux sous-classes sont les suivantes :
\begin{itemize}
\item [C]InitFactualDataReadingProcess[/C] pour préparer la répétition du processus ponctuel de lecture d'un enregistrement
\item [C]ReadFactualDataRecordProcess[/C] pour exécuter cette lecture d'un enregistrement.
\end{itemize}

Il est d'ores et déjà important de noter que la tâche du modélisateur va se limiter à la gestion de l'initialisation, parce que celle-ci va lancer à son tour, et automatiquement, la série des lectures des enregistrements. L'attitude du modélisateur est donc ici identique à celle adoptée pour le processus (continu) de lecture des fichiers de données séquentielles (voir page \pageref{procLectureSeqDF}). En conséquence, le modélisateur va spécifier dans le processus d'initialisation le fichier lu et la méthode d'exploitation des champs des enregistrements (méthode [Y]PostRead[/Y], voir plus loin), puis transfèrera ces spécifications au processus de lecture effective des enregistrements (classe [C]ReadFactualDataRecordProcess[/C]).

 Ces deux processus sont naturellement dotés d'une méthode d'exécution (mobilisée par la directive EXEC). Elles ont dans \Db un corps prédéfini, mais surchargeable.\\

Pour le processus d'initialisation de la lecture, ce corps prédéfini réalise les tâches suivantes :
\begin{itemize}
\item instanciation d'un fichier logique [C]FactualDataFile[/C] et association au fichier physique ciblé
\item ouverture du fichier en lecture et saut des enregistrements d'entête, en nombre renvoyé par la méthode [Y]GetNbTopCommentLines()[/Y]
\item instanciation d'un premier processus de lecture (classe [C]ReadFactualDataRecordProcess[/C]), et transfert vers lui des spécifications du fichier lu et de la méthode d'exploitation des champs des enregistrements
\item programmation pour tout de suite d'un événement exécutant ce processus
\end{itemize}

Quelques remarques communes aux processus de lecture des fichiers de données factuelles et aux processus de lecture des fichiers de données séquentielles sont faites dans le chapitre dédié à ces derniers (notamment sur la possibilité de surcharger les méthodes d'exécution, et sur la manière de spécifier le fichier que doit lire le processus).\\

Pour le processus d'exécution de la lecture, le corps prédéfini de la méthode d'exécution appelle la méthode [Y]ReadNextEnrgt[/Y] de la classe [C]FactualDataFile[/C], sur l'instance associée au processus. Puis, le cas échéant, la méthode qui fait l'objet du paragraphe suivant. \\

Un processus de cette classe est aussi doté d'une méthode d'exploitation des données contenues dans l'enregistrement qui vient d'\^etre lu sur le fichier physique dans le pas courant du processus (et, on le sait, installés dans les "champs" du fichier logique). Cette méthode n'est pas dotée d'un corps prédéfini. Si le développeur veut l'exploiter, il doit obligatoirement définir et écrire la fonction qui deviendra le corps de la méthode (par un appel à [Y]SetPostRead[/Y]). A titre d'exemple, voici un contenu typique de cette fonction :
\begin{verbatim}
[P]void myReadFiles_postReadProcesss_Body(ProcessMethod* pM) {
  ReadFactualDataRecordProcess* pCP 
     = (ReadFactualDataRecordProcess*)pM->DescribedProcess();
  BasicEntity* pE = pCP->ProcessedBasicEntity();
  FactualDataFile* pF = pCP->GetFile();
  float x = pF->GetFloatFieldValue(FIELD_X);
  // la valeur d'un champ est affectée à un descripteur d'une entité
  pE->SetFloatVarValue(X, x);
  ...
}[/P]
\end{verbatim}

Pour la mise en oeuvre d'un processus de cette classe, se reporter à la documentation des processus (continus) de lecture d'un fichier de données séquentielles (voir page \pageref{procLectureSeqDF}).
*/

//@{

/**@name La classe d'initialisation de la lecture des enregistrements
\begin{itemize} \item Directive d'inclusion :  
[P]
\#include CoreDiscreteProcess.h
[/P]
\end{itemize}
*/
//@{
        ///
class InitFactualDataFileReadingProcess : public DiscreteProcess
{
 public:

/**@name LIFECYCLE */
//@{
	/// Constructeur de base. 
  InitFactualDataFileReadingProcess();
       // Constructeur de base sans méthode. Appel exclusif dans RawCopy avec l'argument 1. Les méthodes sont affectées dans RawCopy
  //ReadFactualDataFileProcess(bool); // _no swig_
        /// Constructeur ave affectation du symbole de classe, qui sera celui du fichier lu.
  InitFactualDataFileReadingProcess(UserProcessId, char*);                  
          /// Constructeur par copie profonde, y compris des méthodes.  Invoque le constructeur par copie de la classe-mère.
  InitFactualDataFileReadingProcess(const InitFactualDataFileReadingProcess&); // _no swig_     
                   
          // Constructeur par copie superficielle, c'est-à-dire sans copie des méthodes. La nouvelle instance pointe sur la même méthodesdont le corp code l'exécution du processus. L'entité sur laquelle le processus copié opère un changement d'état est la m\^eme que pour le processus original.\index{méthodes!RawCopy - InitFactualDataFileReadingProcess}    
  //InitFactualDataFileReadingProcess* RawCopy(); 

	/// Destructeur. 
  ~InitFactualDataFileReadingProcess();
                    
          /// Méthode de destruction. Ne détruit pas l'entité sur laquelle le processus opère des changements d'état. Détruit les instances de [C]Method[/C] attachées au processus (précondition, exécution).
   void DeleteInDepth(); // _no swig_
//@}

/**@name [SC]Identification et localisation du fichier lu[/SC] */
//@{
	/// Renvoie le symbole de classe du fichier initialisé par le processus.\index{méthodes!FileClassId}
  UserFileId	FileClassId ();
	/// Affecte le symbole de classe du fichier lu par le processus.
  void   	    FileClassId (UserFileId);

	/// Renseigne le répertoire physique du fichier. C'est le chemin d'accès au fichier relativement au répertoire dans lequel la simulation est exécutée. Le client de ce service doit se charger de la désallocation éventuelle de la chaîne passée en argument.\index{méthodes!SetFilePath}
  void		    SetFilePath (char*);  

	/// Renseigne le nom physique du fichier. C'est le chemin d'accès au fichier relativement au répertoire dans lequel la simulation est exécutée. Le client de ce service doit se charger de la désallocation éventuelle de la chaîne passée en argument.\index{méthodes!SetFullName}
  void		    SetFileFullName (char*);  
  
	/// Renseigne le nom simple du fichier. 
  void		    SetFileSimpleName (char*); // _no swig_ 

	//  Récupère le nom du répertoire, le nom simple et/ou le nom complet du fichier attaché, et affecte au présent processus de lecture.
  void          SetNamesFromFile(FactualDataFile* pF);
  
	// Affecte un nom au fichier associé, à partir du nom du répertoire, du nom simple et/ou du nom complet qui définissent le présent processus de lecture. 
  void          AssignFullNameToFile(FactualDataFile* pF);   

	// Affiche le nom de répertoire, le nom simple et le nom complet spécifiant le fichier attaché au présent processus. 
  void          DisplayNames () const; // _no swig_

        /// Renvoie le nom physique du répertoire du fichier. 
  char*         GetFilePath () const; // _no swig_ 

        /// Renvoie le nom physique du fichier. 
  char*         GetFileFullName () const; // _no swig_   

	/// Renvoie le nom simple du fichier. 
  char*         GetFileSimpleName () const; // _no swig_  

	// Détermine le fichier (SDF) lu par le processus.
  void          SetFile(FactualDataFile*); 

	// Renvoie le pointeur sur le fichier (SDF) lu par le processus.
  FactualDataFile* GetFile () const;

        /// Renvoie vrai si le répertoire du fichier est spécifié. 
  Boolean         PathSpecified () const; // _no swig_ 
        /// Renvoie vrai si le nom complet du fichier est spécifié. 
  Boolean         FullNameSpecified () const; // _no swig_ 
        /// Renvoie vrai si le nom complet du fichier est spécifié. 
  Boolean         SimpleNameSpecified () const; // _no swig_ 
        /// Renvoie vrai si le nom du fichier est spécifié, soit par le nom complet, soit par le répertoire et le nom simple. 
  Boolean         FileNameSpecified () const; // _no swig_ 

         // Dote le processus d'un corps de méthode qui exploite les données lues dans un enregistrement du fichier. \index{méthodes!SetPostRead}
   void            SetPostRead(void (* function)(ProcessMethod*)); // _no swig
//@} 

          // Appelée par le moteur. Affecte la valeur TRUE lors de l'affectation  d'une méthode (par [Y]SetPostRead[/Y]). La valeur FALSE est attribuée par défaut.
  void          HasPostRead(Boolean);   // _no swig_ 
  Boolean       HasPostRead() const;

 protected:
  
  UserFileId    mFileClassId;
  char*		    mFilePath;	      // Nom du répertoire (path)
  char*		    mFileSimpleName;  // Nom simple du fichier
  char*		    mFileFullName;	  // Nom complet du fichier (path)
  FactualDataFile*  mpFile;	  // Pointeur sur l'instance de SDF

  Boolean       mHasPostRead;

};  // _end class_

//@} // fin classe initialisation

/**@name La classe d'exécution de la lecture d'un enregistrement
\begin{itemize} \item Directive d'inclusion :  
[P]
\#include CoreDiscreteProcess.h
[/P]
\end{itemize}
*/
//@{
        ///
class ReadFactualDataRecordProcess : public DiscreteProcess
{
 public:

/**@name LIFECYCLE */
//@{
	/// Constructeur de base. 
  ReadFactualDataRecordProcess();
       // Constructeur de base sans méthode. Appel exclusif dans RawCopy avec l'argument 1. Les méthodes sont affectées dans RawCopy
  //ReadFactualDataFileProcess(bool); // _no swig_
        /// Constructeur ave affectation du symbole de classe, qui sera celui du fichier lu.
  ReadFactualDataRecordProcess(UserProcessId, char*);                  
          /// Constructeur par copie profonde, y compris des méthodes.  Invoque le constructeur par copie de la classe-mère.
  ReadFactualDataRecordProcess(const ReadFactualDataRecordProcess&); // _no swig_     
                   
          // Constructeur par copie superficielle, c'est-à-dire sans copie des méthodes. La nouvelle instance pointe sur la même méthodesdont le corp code l'exécution du processus. L'entité sur laquelle le processus copié opère un changement d'état est la m\^eme que pour le processus original.\index{méthodes!RawCopy - ReadFactualDataRecordProcess}    
  //ReadFactualDataRecordProcess* RawCopy(); 

	/// Destructeur. 
  ~ReadFactualDataRecordProcess();
                    
          /// Méthode de destruction. Ne détruit pas l'entité sur laquelle le processus opère des changements d'état. Détruit les instances de [C]Method[/C] attachées au processus (précondition, exécution).
   void DeleteInDepth(); // _no swig_
//@}

/**@name [SC]Identification et localisation du fichier lu[/SC] */
//@{
	/// Renvoie le symbole de classe du fichier lu par le processus.\index{méthodes!FileClassId}
  UserFileId	FileClassId ();
	/// Affecte le symbole de classe du fichier lu par le processus.
  void   	    FileClassId (UserFileId);

	/// Renseigne le répertoire physique du fichier. C'est le chemin d'accès au fichier relativement au répertoire dans lequel la simulation est exécutée. Le client de ce service doit se charger de la désallocation éventuelle de la chaîne passée en argument.\index{méthodes!SetFilePath}
  void		    SetFilePath (char*);  

	/// Renseigne le nom physique du fichier. C'est le chemin d'accès au fichier relativement au répertoire dans lequel la simulation est exécutée. Le client de ce service doit se charger de la désallocation éventuelle de la chaîne passée en argument.\index{méthodes!SetFullName}
  void		    SetFileFullName (char*);  
  
	/// Renseigne le nom simple du fichier. 
  void		    SetFileSimpleName (char*); // _no swig_ 

	//  Récupère le nom du répertoire, le nom simple et/ou le nom complet du fichier attaché, et affecte au présent processus de lecture.
  void          SetNamesFromFile(FactualDataFile* pF);
  
	// Affecte un nom au fichier associé, à partir du nom du répertoire, du nom simple et/ou du nom complet qui définissent le présent processus de lecture. 
  void          AssignFullNameToFile(FactualDataFile* pF);   

	// Affiche le nom de répertoire, le nom simple et le nom complet spécifiant le fichier attaché au présent processus. 
  void          DisplayNames () const; // _no swig_

        /// Renvoie le nom physique du répertoire du fichier. 
  char*         GetFilePath () const; // _no swig_ 

        /// Renvoie le nom physique du fichier. 
  char*         GetFileFullName () const; // _no swig_   

	/// Renvoie le nom simple du fichier. 
  char*         GetFileSimpleName () const; // _no swig_  

	// Détermine le fichier (SDF) lu par le processus.
  void          SetFile(FactualDataFile*); 

	// Renvoie le pointeur sur le fichier (SDF) lu par le processus.
  FactualDataFile*         GetFile () const;

        /// Renvoie vrai si le répertoire du fichier est spécifié. 
  Boolean         PathSpecified () const; // _no swig_ 
        /// Renvoie vrai si le nom complet du fichier est spécifié. 
  Boolean         FullNameSpecified () const; // _no swig_ 
        /// Renvoie vrai si le nom complet du fichier est spécifié. 
  Boolean         SimpleNameSpecified () const; // _no swig_ 
        /// Renvoie vrai si le nom du fichier est spécifié, soit par le nom complet, soit par le répertoire et le nom simple. 
  Boolean         FileNameSpecified () const; // _no swig_ 

         // Dote le processus d'un corps de méthode qui exploite les données lues dans un enregistrement du fichier. \index{méthodes!SetPostRead}
   void            SetPostRead(void (* function)(ProcessMethod*)); // _no swig

          // Appelée par le moteur. Affecte la valeur TRUE lors de l'affectation  d'une méthode (par [Y]SetPostRead[/Y]). La valeur FALSE est attribuée par défaut.
  void          HasPostRead(Boolean);   // _no swig_ 
  Boolean       HasPostRead() const;
//@}

 protected:
  
  UserFileId    mFileClassId;
  char*		    mFilePath;	      // Nom du répertoire (path)
  char*		    mFileSimpleName;  // Nom simple du fichier
  char*		    mFileFullName;	  // Nom complet du fichier (path)
  FactualDataFile*  mpFile;	  // Pointeur sur l'instance de SDF

  Boolean       mHasPostRead;
};  // _end class_

//@} // fin classe exécution de lecture d'un enregistrement



/**@name Corps de méthodes prédéfinies pour la lecture d'un fichier de données factuelles

Cette méthode de processus est invoquée automatiquement dès lors qu'on a déclaré, et inséré dans l'agenda, un processus de la classe [C]InitFactualDataFileReadingProcess[/C].\\

Au niveau d'une sous-classe de [C]InitFactualDataFileReadingProcess[/C], le modélisateur peut vouloir surcharger son corps. On indique donc ici la signature au niveau de [C]InitFactualDataFileReadingProcess[/C], qui doit être reprise pour la sous-classe (avec un préfixe reprenant le nom de cette sous-classe). L'affectation de ces corps aux méthodes se fait par un appel à SetExec dans le constructeur de la sous-classe.\\

Si cette méthode ne recoit pas un nouveau corps par surchargement, c'est la forme prédéfinie dans le moteur qui est activée.\\

On rappelle ici que ce processus de lecture possède aussi une autre méthode, [Y]PostRead[/Y], dont aucune forme standard de corps n'est incluse dans le moteur. Il revient donc au modélisateur de coder ce corps (sur la base des explications dans la documentation du processus de lecture), et d'affecter ce corps par un appel à [Y]SetPostRead[/Y].
*/
//@{
         ///  Initialisation de la lecture du fichier. Récupère les noms du fichier traité par le processus, et en établit le chemin complet. Puis ouvre le fichier et saute éventuellement le nombre approprié de lignes en début du fichier.
void initFactualDataFileReadingProcess_execProcess_Body(ProcessMethod*); // _no swig_
         // Exécution de la lecture du premier enregistrement non lu du fichier. Invoque le corps d'une éventuelle méthode [Y]PostRead[/Y].
void readFactualDataRecordProcess_execProcess_Body(ProcessMethod*); // _no swig_

//@} // fin corps de méthodes

//@} // fin processus de lecture d'un enregistrement ... suite dans ContinuousProcess.h

#endif  // _COREDISCRETEPROCESS_HH
