//============================================================================
// FILE: CoreException.h
//
// TYPE: C-LIBRARY
// DESCRIPTION: Pseudo-exception catching functions
// 
// AUTHORS: Rellier (INRA/UBIA, Toulouse)
//=X===============X=============================X============================

#ifndef _COREEXCEPTION_HH
#define _COREEXCEPTION_HH

// SYSTEM INCLUDES
//


// PROJECT INCLUDES
//
#include "File.h" // pour FileClassType
#include "CoreTypes.h"
#include "CoreSimTime.h"

void exitOnException(int code);


/**@name CDPFORTEXDescription et gestion des situations d'erreur 
\begin{itemize} \item Directive d'inclusion : 
[P]
\#include CoreException.h
[/P] 
\end{itemize} 
Un certain nombre de méthodes (au sens de C++), dans diverses classes, savent reconnaître des situations particulières (en général indésirables) portant sur leurs paramètres ou sur l'état des entités qu'elles manipulent. Dans une telle situation, la méthode en question {\em peut lever une exception}\index{exception}.\\

 Le langage C++ offre un mécanisme standard de reconnaissance et de traitement d'exception. Il consiste pour le programmeur à déclarer des classes particulières dites {\em classes d'exceptions}\index{exception!classe d'exception}. Dans une situation qu'il a choisie (et qu'il a pris soin de spécifier dans un bloc {\em try}), le programmeur crée une instance d'une classe d'exception appropriée (instruction {\em throw}) en valuant ses attributs en fonction du problème rencontré. L'exploitation (c'est-à-dire le captage d'une exception en cours d'exécution) se fait dans un bloc {\em catch}\index{exception!catch}\index{catch} qui suit le bloc {\em try}\index{exception!try}\index{try} dans lequel a été inséré (directement ou indirectement) l'appel à la méthode qui a levé l'exception. \\

Ce mécanisme standard n'est utilisé dans la bibliothèque \Db que pour capter des erreurs dans la gestion des tableaux (classe [C]Tableau[/C]). Les messages édités lors de ces erreurs sont documentés dans le premier lien dans la liste ci-dessus. On explique aussi comment le développeur d'une application peut exploiter ce mécanisme dans son propre code. \\

Pour toutes les autres erreurs envisagées, les déclenchements d'exceptions par {\em throw} ont été remplacés par des appels à des fonctions globales qui réalisent le même effet. Elles sont de deux types :\\
\begin{itemize}
\item celles qui provoquent l'arrêt du programme ({\em exceptions} proprement dit) : un message d'erreur est alors toujours affiché ;
\item celles qui ne font qu'afficher un {\em avertissement}, et encore seulement si la variable globale [Y]g_BD_TraceWarnings[/Y] a la valeur {\em vrai}.
\end{itemize} 
Les messages édités lors de ces erreurs sont documentés dans les liens  à partir du second, dans la liste ci-dessus. 
 */
//@{ 

//===========================================================================

/**@name Erreurs sur les tableaux
Les services de bas niveau opérant sur des tableaux sont utilisés dans de nombreux services de plus haut niveau : fonctions globales, méthodes des différentes classes ([C]Entity[/C], [C]Process[/C], etc.).\\

Certaines erreurs faites dans la manipulation des tableaux font l'objet d'un message général, non particularisé au contexte dans lequel cette erreur est survenue, dans le code de la bibliothèque \Db ou dans le code applicatif. Ces erreurs sont documentées dans le premier lien ci-dessus.\\

D'autres occasionnent un message dans lequel est indiqué dans quel code, manipulant un tableau, est apparue la situation d'erreur. Ce code peut faire partie de la bibliothèque \Db ou de l'application. Pour permettre une particularisation au contexte d'appel, on utilise le mécanisme standard de captage d'erreur, dont on rappelle ici le schéma :

[P]
\begin{verbatim}
//
// le code réputé pouvoir rencontrer une situation interdite
//
template <class T>
T& Tableau<T>::get_nth(int k) {
  if (k < 0 || k >= nbElements) {
      throw OUT_OF_TAB_RANGE_GET_NTH;
    }
  else 
    ...
}
\end{verbatim}
[/P]
Dans le code qui est réputé pouvoir révéler une situation interdite, ici [Y]get_nth(k)[/Y], son programmeur à lancé, quand la situation interdite est rencontrée, une exception au moyen d'une instruction [Y]throw <e>[/Y].
[P]
\begin{verbatim}
//
// un code applicatif, appelant un service pouvant révéler une situation interdite
//
void UnCodeAppelant() {
  Tableau<int>* tab = new Tableau<int>(0, 10); // un tableau de 10 éléments tous égaux à 0
  try {
    int index = f();  // une fonction retournant un entier
    int k = tab->get_nth(index);
  }
  catch (int e) {
    // une réaction particulière pour la situation interdite
    // la réaction ci-dessous est prédéfinie dans DIESE
    TabIndexOutOfRange(e, "UnCodeAppelant", tab, index, TRUE);
  }
}
\end{verbatim}
[/P]
Dans l'application de plus haut niveau, le code réputé pouvoir révéler une situation interdite, ici [Y]get_nth(index)[/Y], est inclus dans un bloc {\em try}.\\
Ce bloc {\em try} est immédiatement suivi par un bloc {\em catch}, le "capteur d'exception", qui reçoit en argument l'exception lancée (ici OUT_OF_TAB_RANGE_GET_NTH). Le corps du {\em catch} n'a plus qu'à exploiter cette information et le contexte d'appel pour émettre à l'utilisateur un message approprié.\\
DIESE contient un certain nombre de réactions prédéfinies corespondant à des situations d'erreur envisageables. Elles sont documentées dans le second lien dans la liste en haut de la présente page.
 */
//@{ 
//---------------------------------------------------------------------------
/**@name Messages généraux

L'édition de ces messages est suivie de l'arrêt du programme. Le message édité ne donne pas d'indication sur la portion de code applicatif qui a appelé le service qui a rencontré une situation d'erreur.\\\\

{\large {\tt InvalidArraySizeIncrement}} : On utilise une valeur interdite pour l'incrément de la taille d'un tableau. Lors de la construction du tableau, ou lors de l'affectation directe de l'incrément (par [Y]sizeIncrement(int)[/Y]), ou lors de son effacement (par [Y]erase(int)[/Y]).\\
Cet incrément doit être strictement positif.\\ 
[P]
\begin{verbatim}
void main() {
  ...
  int incr = f();  // une fonction retournant un entier
  pIntTab* tab = new pIntTab(incr);  // InvalidArraySizeIncrement si incr < 0
  ...
}
\end{verbatim}
[/P]
{\small {\tt
\begin{verbatim}
>>
!!!============== InvalidArraySizeIncrement ==============
!!! constructeur avec increment sur tableau 90d2f08
    spécification d'un incrément invalide : -1.
\end{verbatim}
}}
\\\\

{\large {\tt InvalidInitialArraySize}} : On construit un tableau de taille inférieure à 1.
[P]
\begin{verbatim}
void main() {
  ...
  int size = f();  // une fonction retournant un entier
  pIntTab* tab = new pIntTab(1, size);  // InvalidInitialArraySize si size < 1
  ...
}
\end{verbatim}
[/P]
\\\\

{\large {\tt TabCopyWithInvalidArraySize}} : On copie le contenu d'un tableau dans un autre, alors que les deux tailles ne sont pas identiques.
[P]
\begin{verbatim}
void main() {
  ...
  pIntTab* tab1 = new pIntTab(1, 8);  // 8 éléments de valeur 1
  pIntTab* tab2 = new pIntTab(2, 9);  // 8 éléments de valeur 2
  t1->copy_from(*t2);  // TabCopyWithInvalidArraySize, parce que 8 != 9
  ...
}
\end{verbatim}
[/P]
 */
//@{
/*
template <class T> 
void InvalidArraySizeIncrement(char*, Tableau<T>*, int, bool);
template <class T> 
void InvalidInitialArraySize(char*, Tableau<T>*, int, bool);
template <class T> 
void TabCopyWithInvalidArraySize(char*, Tableau<T>*, const Tableau<T>&, int, int, bool);
*/
//@} 
//---------------------------------------------------------------------------
/**@name Utilisation du mécanisme standard de captage d'erreur

On liste ici les situations d'erreur captées par le mécanisme {\em throw}/{\em try}/{\em catch}. Une situation d'erreur donnée (par exemple un indice hors limites) peut se rencontrer dans plusieurs services sur les tableaux.\ Et tous les services sur les tableaux ne révèlent pas des situations d'erreur.\\
 
\begin{itemize}
\item L'ensemble des services de bas niveau sur les tableaux dont le code est équipé pour lancer une ou l'autre des exceptions (par {\em throw}) est indiqué dans chaque item ci-dessous. \\
Dans son propre code, le développeur pourra inclure dans un bloc {\em try} les appels à ces services, et seulement ceux-là, s'agissant des tableaux.
\item Les exceptions qui sont lancées (par {\em throw}) sont aussi indiquées dans chaque item ci-dessous. Le développeur a tout intérêt à transmettre cette information à son capteur d'exception ({\em catch}), afin de préciser le message d'erreur.
\item Les capteurs d'exception ({\em catch}) sont à la discrétion du développeur, mais \Db en fournit un pour chaque type d'erreur qui fait l'objet, dans son propre code, d'une émision par {\em throw}.
\end{itemize}

On liste ci-dessous les capteurs prédéfinis. L'édition du message d'erreur y est suivie de l'arrêt du programme. Le message édité donne une indication sur la portion de code applicatif qui a appelé le service en situation d'erreur.\\\\

{\large {\tt TabIndexOutOfRange}} : On fait référence à un élément du tableau, mais avec un indice invalide, c'est-à-dire en dehors de ses limites en mémoire. Les services de bas niveau qui rencontrent cette erreur sont, avec l'exception correspondante :\\
* l'opérateur [Y][]Y/C] : OUT_OF_TAB_RANGE_OP\\
* les méthodes suivantes :\\
[Y]get_nth[/Y] : OUT_OF_TAB_RANGE_GET_NTH\\
[Y]insert[/Y] : OUT_OF_TAB_RANGE_INSERT\\
[Y]remove[/Y] : OUT_OF_TAB_RANGE_REMOVE\\
[Y]remove_from[/Y] : OUT_OF_TAB_RANGE_REMOVE_FROM \\
[Y]remove_to[/Y] : OUT_OF_TAB_RANGE_REMOVE_TO\\
[Y]replace_by_list[/Y] : OUT_OF_TAB_RANGE_REPLACE_L\\ 
[Y]replace_by_item[/Y] : OUT_OF_TAB_RANGE_REPLACE_I\\
[P]
\begin{verbatim}
void UnCodeAppelant() {
  Tableau<int>* tab = new Tableau<int>(0, 10); // un tableau de 10 éléments tous égaux à 0
  try {
    int index = f();  // une fonction retournant un entier
    int k = (*tab)[index];
  }
  catch (int e) { 
    TabIndexOutOfRange(e, "UnCodeAppelant", tab, index, TRUE);
  }
}
\end{verbatim}
[/P]
{\small {\tt
\begin{verbatim}
>>
!!!============== TabIndexOutOfRange ==============
   operator [] : 12 est un indice hors limites pour Tableau 8b60f08 de 10 éléments
   au cours de l'exécution de 'UnCodeAppelant'
!!!============================
\end{verbatim}
}}
\\

{\large {\tt VoidArray}} : On fait référence à un élément d'un tableau qui n'en a pas. Les services de bas niveau qui rencontrent cette erreur sont les méthodes , avec l'exception correspondante :\\
[Y]front[/Y] : VOID_ARRAY_FRONT\\
[Y]back[/Y] : VOID_ARRAY_BACK\\  
[P]
\begin{verbatim}
void UnCodeAppelant() {
  Tableau<int>* tab = new Tableau<int>(); // un tableau encore vide
  try {
    int elem = tab->front();
  }
  catch (int e) {
    VoidArray(e, "UnCodeAppelant", tab, TRUE);
  }
}
\end{verbatim}
[/P]
{\small {\tt
\begin{verbatim}
>>
!!!============== VoidArray ==============
   front : le Tableau 8b60f08 ne possède aucun élément
   au cours de l'exécution de 'UnCodeAppelant'
!!!============================
\end{verbatim}
}}
\\

{\large {\tt MustBePresent}} : On cherche à enlever un élément d'un tableau, mais cet élément réputé présent est absent. Le service de bas niveau qui rencontre cette erreur est la méthode , avec l'exception correspondante :\\
[Y]remove_first[/Y] : MUST_BE_PRESENT_REMOVE_FIRST\\
[P]
\begin{verbatim}
void UnCodeAppelant() {
  Tableau<Entity*>* tab = new Tableau<Entity*>();
  Entity* pA = new Entity();
  Entity* pB = new Entity();
  tab->push_back(pA);
  try {
    tab->remove_first(pB, TRUE);  // TRUE : pB must be present
  }
  catch (int e) {
    MustBePresent(e, "UnCodeAppelant", tab, TRUE);
  }
}
\end{verbatim}
[/P]
{\small {\tt
\begin{verbatim}
>>
!!!============== MustBePresent ==============
   remove_first : le Tableau 8b60f08 ne possède pas l'élément à enlever
   au cours de l'exécution de 'UnCodeAppelant'
!!!============================
\end{verbatim}
}}
 */
//@{ 
/*
 */
//@}
 
//---------------------------------------------------------------------------
//@}
//===========================================================================

/**@name Erreurs sur les descripteurs d'entités */
//@{

//---------------------------------------------------------------------------
/**@name Affectation d'une valeur hors domaine de définition
Exceptions levées lorsqu'on tente d'affecter à un descripteur (constant ou variable) une valeur hors de son domaine de définition (entier, flottant, 'pointeur sur une entité', 'chaîne de caractères'.\\

Le domaine est :
\begin{itemize}
\item un ensemble d'intervalles dans les entiers ou les réels pour les types numériques\\
\item un ensemble de chaînes de caractères pour un type 'chaîne de caractères'\\
\item un ensemble de pointeurs sur des instances d'entités, ou bien un ensemble de classes d'entités, pour un descripteur à valeur 'pointeur sur une entité'.
\end{itemize}

{\large {\tt Descripteur numérique}} : 

[P]
\begin{verbatim}
class ClassA : public Entity {...};
ClassA::ClassA() {
  IntVariableDescriptor* pD = new MyDesc(); AddVariableDescriptor(pD);
  pD->AddSubDomain(0, 5);
  pD->AddSubDomain(10, 15);
  pD->SetValue(8);          // OutOfIntDomain
}
\end{verbatim}
[/P]

{\large {\tt Descripteur '(tableau de) pointeurs sur une entité'}} : 

[P]
\begin{verbatim}
class ClassA : public Entity {...};
class ClassB : public Entity {...};
class ClassC : public Entity {...};
ClassA::ClassA() {
  EntityConstantDescriptor* pD1 = new MyDesc(); AddConstantDescriptor(pD);
  pD1->ValueClassId(CLASS_B);
  ClassC* pC_1 = new ClassC();
  pD1->SetValue(pC_1);                 // OutOfEntityDomain

  EntityTabConstantDescriptor* pD2 = new MyTabDesc(); AddConstantDescriptor(pD);
  pD2->ValueClassId(CLASS_B);
  pD2->AddValueElement(pC_1);          // OutOfEntityDomain
}
\end{verbatim}
[/P]

{\large {\tt Descripteur 'chaîne de caractères'}} : 

[P]
\begin{verbatim}
class ClassA : public Entity {...};
ClassA::ClassA() {
  StringConstantDescriptor* pD = new MyDesc(); AddConstantDescriptor(pD);
  pD->SetSubDomain("jean");
  pD->AddSubDomain("pierre");
  pD->SetValue("louis);          // OutOfStringDomain
}
\end{verbatim}
[/P]
 
*/ 
//@{
void OutOfIntDomain(VariableDescriptor*, int);
void OutOfIntDomain(ConstantDescriptor*, int);
void OutOfFloatDomain(VariableDescriptor*, float);
void OutOfFloatDomain(ConstantDescriptor*, float);
void OutOfDoubleDomain(VariableDescriptor*, double);
void OutOfDoubleDomain(ConstantDescriptor*, double);
void OutOfEntityDomain(VariableDescriptor*, BasicEntity*);
void OutOfEntityDomain(ConstantDescriptor*, BasicEntity*);
void OutOfStringDomain(VariableDescriptor*, char*);
void OutOfStringDomain(ConstantDescriptor*, char*); 
//@}

//---------------------------------------------------------------------------

/**@name Affectation incorrecte d'un domaine de valeurs 'classe d'entités'

{\large {\tt MisuseValueClassId}} : Exception levée lorsqu'on tente de définir par [Y]ValueClassId[/Y] la première (et peut-être seule) classe des valeurs d'un descripteur à valeur 'pointeur sur une entité', alors qu'on en a déjà défini une. Utiliser dans ce cas [Y]AddValueClassId[/Y].\\
[P]
\begin{verbatim}
class ClassA : public Entity {...};
ClassA::ClassA() {
  EntityConstantDescriptor* pD = new MyDesc(); AddConstantDescriptor(pD);
  pD->ValueClassId(CLASS_B);
  pD->ValueClassId(CLASS_C);  // MisuseValueClassId
  pD->AddValueClassId(CLASS_C);
}
\end{verbatim}
[/P]

{\large {\tt MisuseAddValueClassId}} : Exception levée lorsqu'on tente de définir par [Y]AddValueClassId[/Y] une classe de valeurs pour un descripteur à valeur 'pointeur sur une entité', lorsque cette classe ne possède pas d'ancêtre commun (hormis O\_BASIC\_ENTITY) avec toutes les classes déjà autorisées. 
[P]
\begin{verbatim}
class ClassA : public Entity {...};
class ClassB : public Entity {...};
class ClassB1 : public B {...};
class ClassB2 : public B {...};
class ClassB3 : public B {...};
class ClassC : public Entity {...};
ClassA::ClassA() {
  EntityConstantDescriptor* pD = new MyDesc(); AddConstantDescriptor(pD);
  pD->ValueClassId(CLASS_B1);
  pD->AddValueClassId(CLASS_C);  // MisuseAddValueClassId
  pD->AddValueClassId(CLASS_B2); 
}
\end{verbatim}
[/P]
*/
//@{
void MisuseValueClassId(Descriptor*, UserClassId);

void MisuseAddValueClassId(Descriptor*, UserClassId);
//@}

//---------------------------------------------------------------------------

/**@name Usage interdit d'une valeur par défaut
{\large {\tt DefaultValueNotSupported}} : Exception levée lorsqu'on fait référence à une valeur par défaut d'un descripteur alors que la notion de valeur par défaut n'existe pas pour son type. dont le type 

tente de définir par [Y]ValueClassId[/Y] la première (et peut-être seule) classe des valeurs d'un descripteur à valeur 'pointeur sur une entité', alors qu'on en a déjà défini une. Utiliser dans ce cas [Y]AddValueClassId[/Y].\\
[P]
\begin{verbatim}
class ClassA : public Entity {...};
ClassA::ClassA() {
  StringTabConstantDescriptor* pD = new MyDesc(); AddConstantDescriptor(pD);
  if(DefaultValueIsAssigned(pD->ClassSymbol()) {  // DefaultValueNotSupported
    ...
  }
  ...
}
\end{verbatim}
[/P]
*/
//@{
  void DefaultValueNotSupported(const DescriptedEntity*, UserClassId, ValueTypeId);
//@}

//---------------------------------------------------------------------------

/**@name Accès impossible ou interdit à un moniteur
{\large {\tt DoNotHaveMonitorAttached}} : Exception levée lorsqu'on tente d'accéder au démon attaché au descripteur (par [Y]GetMonitor()[/Y]), alors qu'il n'y en a pas.
[P]
\begin{verbatim}
  IntDescValueMonitor* MyDescMonitor = 
        new IntDescValueMonitor("myDescMonitor");
  MyDescMonitor->AssignWhenSetIntMethod(myDesc_whenSetInt_Body);

class ClassA : public Entity {...};
ClassA::ClassA() {
  IntVariableDescriptor* pD = new MyDesc(); // symbole de classe : MY_DESC
  AddVariableDescriptor(pD);
  //MyDescMonitor->AttachToDescriptor(this, MY_DESC);
  DescValueMonitor* pDVM = pD->GetMonitor();  // DoNotHaveMonitorAttached
  pDVM->ExecIntCoreMethod(WHEN_SET_INT);
}
\end{verbatim}
[/P]

{\large {\tt MonitorNotSupportedForEraseEntityTabDescriptor}} : Avertissement émis lorsqu'on tente d'effacer le tableau de valeurs pour un descripteur de ce type, alors qu'on a défini un démon sur changement de valeur pour ce descripteur. Cette fonctionnalité n'est pas supportée par la présente version de \D.
[P]
\begin{verbatim}
  EntityTabDescValueMonitor* MyDescMonitor = 
        new EntityTabDescValueMonitor("myDescMonitor");
  MyDescMonitor->AssignWhenAddEntityMethod(myDesc_whenAddEntity_Body);

class ClassA : public Entity {...};
ClassA::ClassA() {
  EntityTabVariableDescriptor* pD = new MyDesc(); // classId : MY_DESC 
  AddVariableDescriptor(pD);
  MyDescMonitor->AttachToDescriptor(this, MY_DESC);
  pD->ClearValues(); // MonitorNotSupportedForEraseEntityTabDescriptor
}
\end{verbatim}
[/P]


{\large {\tt MonitorNotSupportedForEraseDescValueSpecTabDescriptor}} : Avertissement émis lorsqu'on tente d'effacer le tableau de valeurs pour un descripteur qui a ce type, alors qu'on a défini un démon sur changement de valeur pour ce descripteur. Cette fonctionnalité n'est pas supportée par la présente version de \D.

[P]
\begin{verbatim}
  DescValueSpecTabDescValueMonitor* MyDescMonitor = 
        new DescValueSpecTabDescValueMonitor("myDescMonitor");
  MyDescMonitor->AssignWhenAddEntityMethod(myDesc_whenAddDescValueSpec_Body);

class ClassA : public Entity {...};
ClassA::ClassA() {
  DescValueSpecTabVariableDescriptor* pD = new MyDesc(); // classId : MY_DESC
  AddVariableDescriptor(pD);
  MyDescMonitor->AttachToDescriptor(this, MY_DESC);
  pD->ClearValues(); // MonitorNotSupportedForEraseDescValueSpecTabDescriptor
}
\end{verbatim}
[/P] 
*/
//@{
void DoNotHaveMonitorAttached(const VariableDescriptor*);
void MonitorNotSupportedForEraseEntityTabDescriptor(Descriptor*);
void MonitorNotSupportedForEraseDescValueSpecTabDescriptor(Descriptor*);
//@}

//---------------------------------------------------------------------------

/**@name Echec d'attachement d'un moniteur
   {\large {\tt AttachmentFailed}} : Exception levée lorsqu'on tente d'attacher, à un descripteur d'une entité désignée, un démon non alloué.
[P]
\begin{verbatim}
//IntDescValueMonitor* MyDescMonitor = 
//      new IntDescValueMonitor("myDescMonitor");
//MyDescMonitor->AssignWhenSetIntMethod(myDesc_whenSetInt_Body);

class ClassA : public Entity {...};
ClassA::ClassA() {
  IntVariableDescriptor* pD = new MyDesc(); // symbole de classe : MY_DESC
  AddVariableDescriptor(pD);
  MyDescMonitor->AttachToDescriptor(this, MY_DESC);// AttachmentFailed
}
\end{verbatim}
[/P]

{\large {\tt DirectAttachmentFailed}} : Exception levée lorsqu'on tente d'attacher, directement à un descripteur désigné, un démon non alloué.
[P]
\begin{verbatim}
//IntDescValueMonitor* MyDescMonitor = 
//      new IntDescValueMonitor("myDescMonitor");
//MyDescMonitor->AssignWhenSetIntMethod(myDesc_whenSetInt_Body);

class ClassA : public Entity {...};
ClassA::ClassA() {
  IntVariableDescriptor* pD = new MyDesc(); // symbole de classe : MY_DESC
  AddVariableDescriptor(pD);
  MyDescMonitor->AttachToDescriptor(pD);// DirectAttachmentFailed
}
\end{verbatim}
[/P]


{\large {\tt AttachmentTypeError}} : Exception levée lorsqu'on tente d'attacher un démon d'un type différent de celui du descripteur.

[P]
\begin{verbatim}
IntDescValueMonitor* MyDescMonitor = 
      new IntDescValueMonitor("myDescMonitor");
MyDescMonitor->AssignWhenSetIntMethod(myDesc_whenSetInt_Body);

class ClassA : public Entity {...};
ClassA::ClassA() {
  FloatVariableDescriptor* pD = new MyDesc(); // symbole de classe : MY_DESC
  AddVariableDescriptor(pD);
  MyDescMonitor->AttachToDescriptor(this, MY_DESC); // AttachmentTypeError
}
\end{verbatim}
[/P] 
*/
//@{
void AttachmentFailed(VariableDescriptor* pVD, DescriptedEntity* pObj);
void DirectAttachmentFailed(VariableDescriptor* pVD);
void AttachmentTypeError(VariableDescriptor*, DescriptedEntity*, char*);
//@}
//---------------------------------------------------------------------------

/**@name Assignation interdite entre descripteurs

{\large {\tt UnAuthorizedAssignment}} : Exception levée lorsqu'on tente d'effectuer une opération d'assignation (opérateur d'égalité) entre deux descripteurs non typés.
[P]
\begin{verbatim}
class ClassA : public Entity {...};
ClassA::ClassA() {
  Descriptor* pD1 = new MyDesc1();  AddVariableDescriptor(pD);
  Descriptor* pD2 = new MyDesc2();  AddVariableDescriptor(pD);
  *pD1 = *pD2;              // UnAuthorizedAssignment

  IntVariableDescriptor* pIntD1 = new MyIntDesc1();  AddVariableDescriptor(pD);
  pIntD1->SetValue(8);
  IntVariableDescriptor* pIntD2 = new MyIntDesc2();  AddVariableDescriptor(pD);
  pIntD2->SetValue(0);
  *pIntD2 = *pIntD1;
  printf("\nLa valeur de pIntD2 est %d.", pIntD2->GetIntValue());
}
\end{verbatim}
[/P]
\begin{verbatim}
>>
La valeur de pIntD2 est 8.
\end{verbatim}

*/
//@{
void UnAuthorizedAssignment(const Descriptor*, const Descriptor*);
//@}

//@}  // fin descripteurs  

//===========================================================================

/**@name Erreurs sur les entités */
//@{ 

/**@name Erreurs commises sur la structure supposée en composants

{\large {\tt ComponentCannotCoexistWithElement}} : Exception levée lorsqu'on tente de déclarer un symbole de classe pour un composant, alors qu'on a déjà déclaré un symbole de classe pour les éléments. 

{\large {\tt UnAuthorizedAccessToElement}} : Exception levée lorsqu'on tente d'accéder à des éléments de l'entité, alors qu'elle est construite pour recevoir des composants.

{\large {\tt ComponentAlreadyPlugged}} : Exception levée lorsqu'on tente, par [Y]AddComponent[/Y], d'ajouter un composant une entité, alors qu'une autre entité de la même classe est déjà installé comme composant.

{\large {\tt ComponentPluggedElsewhere}} : Exception levée lorsqu'on tente, par [Y]AddComponent[/Y], d'ajouter un composant une entité, alors qu'il est déjà composant d'une autre.

{\large {\tt BadComponentClassId}} : Exception levée lorsqu'on cherche dans le tableau des composants d'une entité, par [Y]GetComponent[/Y], une entité de classe non représentée.

{\large {\tt NotYetPluggedComponent}} : Exception levée lorsqu'on cherche dans le tableau des composants d'une entité, par [Y]GetComponent[/Y], un composant non encore installé.

{\large {\tt BadComponentName}} : Exception levée lorsqu'on cherche, par [Y]GetComponent[/Y] avec son nom d'instance, dans le tableau des composants d'une entité, une entité non présente.

{\large {\tt DoNotHaveSuperEntity}} : Exception levée lorsqu'on cherche, par [Y]GetSuperEntity[/Y], à accéder à la super-entité d'une entité, alors que celui-ci n'en possède pas.

{\large {\tt SuperEntityAlreadyEquippedWithClass}} : Exception levée lorsqu'on tente, par [Y]SetSuperEntity[/Y], de désigner comme super-entité d'un composant une entité qui possède déjà un composant de la même classe.

 */
//@{
void ComponentCannotCoexistWithElement(BasicEntity*, UserClassId);
void UnAuthorizedAccessToElement(const BasicEntity*, char*);
void ComponentAlreadyPlugged(BasicEntity* container, 
                             BasicEntity* inPlace, 
                             BasicEntity* candidate);
void ComponentPluggedElsewhere(BasicEntity* container, 
                               BasicEntity* otherContainer,
                               BasicEntity* candidate);
void BadComponentClassId(BasicEntity*, UserClassId);
void NotYetPluggedComponent(BasicEntity*, UserClassId);
void BadComponentName(BasicEntity*, char*);
void DoNotHaveSuperEntity(BasicEntity*, UserClassId);
void SuperEntityAlreadyEquippedWithClass(BasicEntity*, UserClassId, 
                                         BasicEntity*, BasicEntity*);
//@}




/**@name Erreurs commises sur la structure supposée en éléments  

{\large {\tt ElementCannotCoexistWithComponent}} : Exception levée lorsqu'on tente de déclarer un symbole de classe pour les éléments, alors qu'on a déjà déclaré un ou plusieurs symboles de classe pour des composants. 

{\large {\tt UnAuthorizedAccessToComponent(BasicEntity*, char*)}} : Exception levée lorsqu'on tente d'accéder à des composants de l'entité, alors qu'elle est construite pour recevoir des éléments. 

{\large {\tt UnAuthorizedAccessToComponent(BasicEntity*, char*, UserClassId)}} : Exception levée lorsqu'on tente d'accéder à un composant de l'entité, alors qu'elle est construite pour recevoir des éléments. On précise le symbole du composant accédé.            

{\large {\tt BadSubEntityClass}} : Exception levée lorsqu'on ajoute par [Y]AddElement[/Y] ou [Y]AddComponent[/Y], une entité de classe incorrecte, ou bien lorsqu'on retire par [Y]RemoveComponent[/Y] une entité de classe incorrecte ou qui n'est pas encore composant.   

{\large {\tt AddElementFailed}} : Avertissement lancé lorsqu'on tente d'ajouter sans succès par [Y]AddElement[/Y] une entité (second argument) comme élément d'une autre (premier élément).   

{\large {\tt RemoveElementFailed}} : Avertissement lancé lorsqu'on tente de retirer sans succès par [Y]RemoveElement[/Y] une entité (second argument) de l'ensemble des éléments d'une autre (premier élément).   

{\large {\tt ReplaceElementFailed}} : Avertissement lancé lorsqu'on tente par [Y]ReplaceElement[/Y] de remplacer sans succès, dans la liste des éléments d'une entité, une entité (premier argument)  par un ensemble d'entités. 

{\large {\tt BadSuperSetClass}} : Exception levée lorsqu'on tente de désigner comme sur-ensemble d'une entité, par [Y]AddSuperSet[/Y], une entité de classe incorrecte. 

{\large {\tt NoElement}} : Avertissement émis lorsqu'on cherche un élément d'une entité, supposer exister, alors qu'elle n'en a pas. 

{\large {\tt NoRankedElement}} : Avertissement émis lorsqu'on cherche un élément de rang k d'une entité, supposer exister, alors que cet indice n'est pas représenté.

{\large {\tt NoNamedElement}} : Avertissement émis lorsqu'on cherche un élément de nom (d'instance ou de classe) donné d'une entité, supposer exister, alors qu'il n'est pas élément. 

{\large {\tt NoClassNamedElement}} : Avertissement émis lorsqu'on cherche un élément de symbole de de classe donné d'une entité, supposer exister, alors qu'il n'est pas élément. 

{\large {\tt NoSuperSet}} : Avertissement émis lorsqu'on cherche un sur-ensemble pour une entité qui n'en a pas.
 */
//@{  
void ElementCannotCoexistWithComponent(BasicEntity*, UserClassId);
void UnAuthorizedAccessToComponent(const BasicEntity*, char*);
void UnAuthorizedAccessToComponent(const BasicEntity*, char*, UserClassId id);
void BadSubEntityClass(BasicEntity*, UserClassId, BasicEntity*);
void AddElementFailed(BasicEntity*, BasicEntity*);
void RemoveElementFailed(BasicEntity*, BasicEntity*);
void ReplaceElementFailed(BasicEntity*, BasicEntity*);
void BadSuperSetClass(BasicEntity*, UserClassId, BasicEntity*);
void NoElement(const BasicEntity*);
void NoRankedElement(const BasicEntity*, int k);
void NoNamedElement(const BasicEntity*, char*);
void NoClassNamedElement(const BasicEntity*, UserClassId);
void NoSuperSet(const BasicEntity*);
//@}


/**@name Erreurs d'attachement d'un moniteur sur la structure

{\large {\tt StructAttachmentFailed}} : Exception levée lorsqu'on tente d'attacher un démon non alloué.

{\large {\tt DoNotHaveMonitorAttached}} : Exception levée lorsqu'on tente d'accéder au démon attaché à la structure de l'entité (par [Y]GetMonitor()[/Y]), alors qu'il n'y en a pas.
 */
//@{
void StructAttachmentFailed(BasicEntity* pObj);
void DoNotHaveMonitorAttached(const BasicEntity*);
//@}



/**@name Accès incorrect aux descripteurs

{\large {\tt InexistantVariableDescriptor}} : Exception levée lorsque le descripteur variable cherché par [Y]GetVariableDescriptor[/Y] n'est pas présent dans le tableau.

{\large {\tt InexistantConstantDescriptor}} : Exception levée lorsque le descripteur constant cherché par [/Y]GetConstantDescriptor[/Y] n'est pas présent dans le tableau.

{\large {\tt BadDescriptorValueType}} : Exception levée lorsqu'on accède ou qu'on affecte une valeur à un descripteur, par une méthode d'accès qui ne sait pas traiter le type de valeur en question. Par exemple, accès à une valeur de type {\small {\tt int}} par [Y]GetFloatValue[/Y].

{\large {\tt InexistantDescriptor}} : Exception levée lorsque le descripteur, cherché par [Y]GetDescriptor[/Y] ou bien objet d'une affectation de valeur ou d'un accès à la valeur, n'est présent ni dans le tableau des descripteurs constants ni dans le tableau des descripteurs variables.

{\large {\tt BadDescriptorTypeWhenAdd}} : Exception levée lorsqu'on tente d'ajouter, par [Y]AddVariableDescriptor[/Y], un descripteur constant, ou {\small {\tt vice-versa}}.
 */
//@{
void InexistantVariableDescriptor(const DescriptedEntity*, UserDescriptorId);
void InexistantConstantDescriptor(const DescriptedEntity*, UserDescriptorId);
void BadDescriptorValueType(const DescriptedEntity*, char*, char*);
void InexistantDescriptor(const DescriptedEntity*, UserDescriptorId, char*);
void BadDescriptorTypeWhenAdd(DescriptedEntity*, Descriptor*);
//@}


/**@name Accès incorrect aux méthodes

{\large {\tt BadMethodValueType}} : Exception levée lorsqu'on tente une opération (affectation d'un corps, exécution du corps) sur une méthode dont le type ne correspond pas aux données passées en paramètres (par exemple message [Y]ExecIntMethod[/Y] à une méthode de type {\em float}). 

{\large {\tt InexistantMethod}} : Exception levée lorsque la méthode cherchée par [Y]GetMethod[/Y] n'est pas présente dans le tableau des méthodes. 

{\large {\tt BadMethodTypeWhenAdd}} : Exception levée lorsqu'on ajoute une méthode qui est une méthode de processus ou d'événement. 
 */
//@{
void BadMethodValueType(Entity*, Method*, char*);
void InexistantMethod(Entity*, UserMethodId, char*);
void BadMethodTypeWhenAdd(Entity*, Method*);
//@}


/**@name Recherche infructueuse d'une classe ou d'instances

{\large {\tt UnknownEntityClass}} : Exception levée lorsqu'on recherche une classe d'entité (en fait la métaclasse associée) par un entier associé sémantiquement à un symbole, quand ce nombre entier n'est justement encore associé à aucun symbole de classe.

{\large {\tt BadEntityClassSpecification}} : Exception levée lorsqu'on recherche une classe d'entité (en fait la métaclasse associée) par son symbole, ou som nom traduit en symbole, quand ce symbole de classe n'a pas une valeur admissible, ou n'a pas pu être déterminé.

{\large {\tt UnfoundEntityBasedOnInstanceName}} : Exception levée lorsqu'on recherche une entité à partir de son nom d'instance, alors qu'aucune entité n'existe avec ce nom. L'opération en cours est identifiée par le second argument.

{\large {\tt EmptyInstanceListForMetaClass}} : Avertissement émis lorsqu'on recherche par [Y]GetFirstEntity[/Y], une entité de symbole de classe donné, alors que la liste pour cette classe est vide. Intervient notamment pour la classe [C]Entity[/C] et ses ascendantes, pour lesquelles le moteur ne maintient pas une liste d'instances créées.

 */
//@{  
void UnknownEntityClass(UserClassId, char*); 
void UnknownEntityClass(UserClassId, char*, const BasicEntity*);  
void BadEntityClassSpecification(char*);  
void BadEntityClassSpecification(char*, const BasicEntity*);
																				void UnfoundEntityBasedOnInstanceName(char* service, char* name);
void EmptyInstanceListForMetaClass(UserClassId id);

         // Exception levée lorsqu'on recherche par [Y]GetInstance[/Y], adressé à une classe donnée, l'entité de rang {\small {\tt k}}, alors que la liste pour cette classe est vide. Intervient notamment pour la classe [C]Entity[/C] et ses ascendantes.
void EmptyInstanceListForMetaClass(UserClassId id, int k);
//@} 


/**@name Sources diverses d'erreur sur les entités

{\large {\tt SimulationNotYetCreated}} : Exception levée pour imposer que soit créée une instance de [C]Simulation[/C] avant qu'on tente d'instancier une quelconque classe d'entité.

{\large {\tt UnknownClassName}} : Exception levée lorqu'on tente d'instancier une classe de nom donné par une fonction d'instanciation d'entités (du type [Y]App_NewEntityFromString[/Y]), alors que ce nom de classe n'est pas connu de cette fonction.

{\large {\tt ReservedUserClassId}} : Exception levée lorsqu'on tente de donner à une entité un symbole de classe réservé par DIESE. Le nombre entier correspondant au symbole doit être supérieur à la valeur de la variable globale [Y]g_BD_ReservedUserClassId[/Y].

{\large {\tt MetaClassNotYetCreated}} : Exception levée lorsqu'on recherche des informations sur une classe de symbole donné qui n'a pas encore reçu d'instances.
 */
//@{
void SimulationNotYetCreated();
void UnknownClassName(char* typeObjet, char* className);
void ReservedUserClassId(UserClassId id);
void UnAuthorizedMultipleInheritance(const BasicEntity*);
void MetaClassNotYetCreated(UserClassId id, char* service);
//@}

//@}  // fin entités    

//===========================================================================

/**@name Erreurs sur les événements

{\large {\tt UnknownClassName}} : Exception levée lorqu'on tente d'instancier une classe de nom donné par une fonction d'instanciation d'entités (du type [Y]App_NewEventFromString[/Y]), alors que ce nom de classe n'est pas connu de cette fonction.

{\large {\tt UnspecifiedProcess}} : Exception levée lorqu'on a instancié un événement sans avoir spécifié le ou les processus qu'il gère.

{\large {\tt TwoEventsWithSameDateAndPriority}} : Avertissement émis lorsque deux événements partagent la même date d'occurrence et la même priorité. Le premier est supposé intervenir avant le second.

{\large {\tt BadMethodValueType}} : Exception levée lorsqu'on tente une opération (affectation d'un corps, exécution du corps) sur une méthode dont le type ne correspond pas aux données passées en paramètres (par exemple message [Y]ExecIntMethod[/Y] à une méthode de type {\em float}).

{\large {\tt UnexpectedProbability}} : Exception levée lorsqu'on tente de construire une instance d'événement avec une probabilité d'occurrence < 0 ou > 1.

{\large {\tt UnexpectedPriorityDegree}} : Exception levée lorsqu'on tente de construire une instance d'événement avec un degré de priorité < 0 ou > 99.

{\large {\tt UnauthorizedMaxOccurenceClockTime}} : Exception levée lorsqu'on affecte une date au plus tard pour l'occurrence de l'événement, alors que la date au plus tôt n'a pas été indiquée.

{\large {\tt UnexpectedDelayNextEvent}} : Exception levée lorsqu'on spécifie un délai d'occurrence pour un événement autogénéré, alors qu'aucune méthode n'est attachée à l'événement pour effectuer cette autogénération.

{\large {\tt UnspecifiedPredefinedMethod}} : Exception levée lorsqu'on a déclaré qu'une méthode de post-conséquence ou d'autogénération était attachée à l'événement, mais que cet attachement n'a pas été effectivement réalisé.
*/
//@{
void UnspecifiedProcess(Event* pEv);
void TwoEventsWithSameDateAndPriority(Event* pEv1, Event* pEv2, int oct1);
void BadMethodValueType(Event*, Method*, char*);
void UnexpectedProbability(Event*, float);
void UnexpectedPriorityDegree(Event*, short);
void UnauthorizedMaxOccurenceClockTime(Event*, int); 
void UnexpectedDelayNextEvent(Event*, int);
																				void UnspecifiedPredefinedMethod(char* type, Event* pEv);
//@}   

//===========================================================================

/**@name Erreurs sur les processus

{\large {\tt UnknownClassName}} : Exception levée lorqu'on tente d'instancier une classe de nom donné par une fonction d'instanciation d'entités (du type [Y]App_NewEventFromString[/Y]), alors que ce nom de classe n'est pas connu de cette fonction.

{\large {\tt DoNotHaveProcessedEntity}} : Avertissement émis lorsqu'on tente d'accéder à l'entité sur laquelle porte le processus, alors qu'elle n'a pas encore été désignée.

{\large {\tt BadProcessedEntityClass}} : Exception levée lorsqu'on tente d'affecter comme entité touchée par le processus, une entité d'une classe différente de celle autorisée par [Y]ProcessedEntityClassId[/Y] et qui n'en descend pas.

{\large {\tt BadProcessedEntitySpec}} : Exception levée lorsqu'on tente de spécifier l'entité touchée par le processus, par une spécification d'ensemble d'entités illégale : définie par une classe différente de celle autorisée par [Y]ProcessedEntityClassId[/Y] et qui n'en descend pas, ou bien dont le mode n'est pas RETURN\_INSTANCES, ou bien qui demande plus d'une entité.

{\large {\tt EntitySpecForProcessedEntityYieldsNone}} : Exception levée lorsqu'on tente d'établir l'entité touchée par le processus par une spécification d'ensemble d'entités dont l'expansion ne renvoie aucune entité. 

{\large {\tt BadMethodValueType}} :  Exception levée lorsqu'on tente une opération (affectation d'un corps, exécution du corps) sur une méthode dont le type ne correspond pas aux données passées en paramètres (par exemple message [Y]ExecIntMethod[/Y] à une méthode de type {\em float}).

{\large {\tt InexistantInitializationMethod}} : Exception levée lorsqu'on tente d'initialiser un processus continu alors qu'aucune méthode n'a été affectée pour cela. 

{\large {\tt InexistantProceedMethod}} : Exception levée lorsqu'on s'apprête à faire avancer un processus continu alors qu'aucune méthode n'a été affectée pour cela.

{\large {\tt InexistantStopMethod}} : Exception levée lorsqu'on tente de stopper un processus continu alors qu'aucune méthode n'a été affectée pour cela. 
*/
//@{

void DoNotHaveProcessedEntity(const Process* pP);
void BadProcessedEntityClass(Process* pP, BasicEntity* pObj);
void BadProcessedEntitySpec(Process* pP, UserClassId id);
void EntitySpecForProcessedEntityYieldsNone(Process* pP, EntitySpec* pES);
void BadMethodValueType(Process*, Method*, char*);
void InexistantInitializationMethod(ContinuousProcess* pP);
void InexistantProceedMethod(ContinuousProcess* pP);
void InexistantStopMethod(ContinuousProcess* pP);
//@}      

//===========================================================================

/**@name Erreurs sur les directives données aux processus dans les événements

{\large {\tt UnexpectedAddToListProcessAction}} : Exception levée lors de l'addition d'un couple {processus, action} par [Y]AddProcessAction[/Y], si l'action est une deuxième de type PROCEED, ou si l'addition du couple ne respecterait pas les critères d'ordre suivants :\begin{itemize}\item	1) INIT avant EXEC avant STOP avant PROCEED ;\item	2) plus petit PAS d'abord, pour deux processus continus.\end{itemize} 

{\large {\tt UnexpectedInsertInListProcessAction}} : Exception levée lorsqu'une deuxième action de type PROCEED est insérée dans la liste des couples {processus, action}, par [Y]InsertProcessAction[/Y].

{\large {\tt UnexpectedVoidListProcessAction}} : Avertissement émis lorsqu'on accède à la liste des couples {processus, action} d'un événement (par [Y]FirstProcess[/Y] ou [Y]FirstActionLabel[/Y]), alors que celle-ci est vide. L'utilisateur doit gérer le pointeur NULL renvoyé par [Y]FirstProcess[/Y] ou la valeur UNSPECIFIED renvoyée par [Y]FirstActionLabel[/Y].

{\large {\tt UnfoundProcessForAddingToEvent}} : Avertissement émis lors d'un ajout de couple {processus, action} avec un processus non trouvé à partir de son nom de classe 

{\large {\tt UnfoundProcessInEvent}} : Exception levée lors de la recherche d'un processus, à partir de son symbole de classe, dans la liste de couples {processus, action}.

{\large {\tt UnAuthorizedDirective}} : Exception levée lorsque la méthode [Y]ApplyConsequences[/Y] tente une directive non autorisée sur un processus. Le premier argument est le processus, le second argument est la directive.
 */
//@{
void UnexpectedAddToListProcessAction(Event*, Process*, ActionType);
void UnexpectedInsertInListProcessAction(Event*, Process*, ActionType);
void UnexpectedVoidListProcessAction(Event*);
void UnfoundProcessForAddingToEvent(char*, ActionType);
void UnfoundProcessInEvent(Event*, UserProcessId);  
void UnAuthorizedDirective(Process*, char*);
//@}   

//===========================================================================

/**@name Erreurs sur les méthodes 

{\large {\tt InexistantArgument}} : Exception levée lorsque l'argument cherché par [Y]GetArg[/Y] n'est pas présent dans le tableau des arguments de la méthode.

{\large {\tt BadArgumentType}} : Exception levée lorsqu'on tente une opération (accès à la valeur, affectation d'une valeur à un argument) sur un argument dont le type ne correspond pas aux données passées en paramètre : par exemple, appel [Y]GetIntValue[/Y] avec en argument le symbole de classe d'un argument à valeur flottante.

{\large {\tt UnsupportedArgumentType}} : Exception levée lors d'une copie de méthode par l'opérateur [Y][][/Y], lorsqu'un argument de la méthode originale n'est pas d'un des types autorisés pour cette méthode.
*/
//@{
void InexistantArgument(Method*, UserArgumentId);
void BadArgumentType(UserArgumentId, UserMethodId, char*);
void UnsupportedArgumentType(char* meth, Argument*);
//@}   

//===========================================================================

/**@name Erreurs sur l'objet 'simulation'  

{\large {\tt BadClockTimeUnit}} : Exception levée quand on a déclaré une unité de temps non gérée par le système.

{\large {\tt NoSimulatedEntity}} : Exception levée lorsqu'on fait référence à l'entité simulée, alors qu'on n'en a pas encore attaché une à la simulation en cours.

{\large {\tt BadClockValueForTm}} : Exception levée lorsqu'on invoque [Y]ClockToTm[/Y] avec en argument une valeur d'horloge négative.
*/
//@{ 
void BadClockValueForTm(int);
void BadClockTimeUnit(TimeUnit);
void NoSimulatedEntity();
//@}

//===========================================================================

/**@name Erreurs sur les spécifications d'ensembles d'entités  

{\large {\tt NoInformationForReferenceClass}} : Exception levée lorsqu'on recherche sans succès le symbole de la classe de référence d'une spécification d'ensemble d'entités, parce que ...\begin{itemize}\item ... ni symbole ni nom de classe de référence n'ont été attribués à la spécification d'ensemble d'entités,\item ... ou bien parce qu'on a attribué un nom de classe de référence, mais que le symbole de classe ne peut pas être inféré parce qu'aucune instance de cette classe n'a encore été créée.\end{itemize} Cet échec peut notamment intervenir lors de l'expansion de la spécification.    

{\large {\tt BadQuantitySpecification}} : Exception levée lorsqu'on tente de créer une spécification d'ensembles d'entités avec des valeurs incompatibles de quantificateur et d'effectif.

{\large {\tt NoReturnInstanceProcedure}} : Exception levée lorsqu'une spécification d'ensembles d'entités de type RETURN_INSTANCES n'est dotée ni d'un prédicat de sélection, ni d'une fonction qui crée une liste d'entités. 

{\large {\tt NoInstantiationProcedure}} : Exception levée lorsqu'une spécification d'ensembles d'entités de type INSTANTIATE n'a pas été dotée d'une fonction qui crée une entité ou une liste d'entités. 
 */
//@{

void NoInformationForReferenceClass(EntitySpec*);
void NoInformationForReferenceClass(EntitySpec*, char*);
void NoInformationForReferenceClass(EntitySpec*, Entity*);

void BadQuantitySpecification(UserClassId, QuantificatorId, int);  
void NoReturnInstanceProcedure(char*, UserClassId);   // + JPR 070906
void NoInstantiationProcedure(char*, UserClassId); // + JPR 070906
//@}   

//===========================================================================

/**@name Erreurs sur les spécifications de domaines de valeurs  

{\large {\tt BadRelationalOperator}} :  Exception levée lorsqu'on tente de construire une spécification de domaine de valeurs avec un opérateur relationnel incorrect.   

{\large {\tt UnAuthorizedEntitySubDomain}} : Exception levée lorsqu'on tente d'ajouter à une spécification de domaine de valeur un élément de type 'pointeur sur une entité', alors qu'il faut indiquer son nom d'instance.  

{\large {\tt UnAuthorizedEntitySubDomain}} : Exception levée lorsqu'on tente d'ajouter à une spécification de domaine de valeur une entité indiquée par son nom d'instance, alors qu'il faut passer le 'pointeur' sur l'entité.
 */
//@{  
void BadRelationalOperator(UserDescriptorId, RelationalOperator); 
void UnAuthorizedEntitySubDomain(DescValueSpec*, BasicEntity*);
void UnAuthorizedEntitySubDomain(DescValueSpec*, UserClassId);
//@}    

//===========================================================================

/**@name Erreurs sur les paramètres  

{\large {\tt InexistantTypedParameter}} : Exception levée lorsqu'on accède à la valeur, ou lorsque'on donne une valeur à un paramètre inexistant.  

{\large {\tt InexistantParameter}} : Exception levée lorsqu'on recherche une instance de la classe [C]Parameter[/C], spécifiée par ses identifiants, mais inexistante.

Le plus souvent, ces erreurs sont causées par l'absence d'affectation de valeur à ce paramètre dans le fichier des paramètres du système, alors que le code du simulateur requiert cette valeur.

{\large {\tt BadDateParameterValue}} : Exception levée lorsque une date spécifiée en paramètre du système (*.par) est invalide.

 */
//@{
void InexistantTypedParameter(char* id1, char* id2, char* type, char* op);
void InexistantParameter(char* id1, char* id2);
void BadDateParameterValue(char* id1, char* id2);
//@}    

//===========================================================================

/**@name Erreurs sur les sources de données externes 

{\large {\tt UnknownFileClassSymbol}} : Exception levée lorqu'on tente d'instancier une classe de fichier séquentiel de symbole de classe donné, par une fonction d'instanciation (du type [Y]App_NewSerialDataFileFromId[/Y]), alors que ce symbole n'est pas connu de cette fonction. Pour exploiter cette exception dans le code de l'application, valuer les deux arguments {\small {\tt fileType}} et {\small {\tt fileId}}. Par exemple : [P]UnknownFileClassSymbol("SequentialDataFile", WEATHER_FILE);[/P].

{\large {\tt BadFileOpenMode}} : Exception levée lorsque tente d'ouvrir un fichier physique avec un mode inconnu (c'est-à-dire ni READ ni WRITE).  

{\large {\tt BadFieldValueType}} : Exception levée lorsqu'on essaie d'affecter ou de récupérer une valeur, celle d'un champ d'enregistrement de fichier, dont le type est incompatible avec le type du champ en question.  

{\large {\tt BadFileName}} : Exception levée lorsqu'on tente d'ouvrir un fichier qui n'existe pas physiquement sur un support mémoire. 

{\large {\tt BadFileType}} : Exception levée lorsqu'on tente d'accéder à une instance de la classe des fichiers ([C]File[/C]) comme si c'était un fichier de données séquentielles, alors qu'il n'est pas de ce type.  

{\large {\tt BadSearchByFileId}} : Exception levée lorsqu'on tente d'accéder à une instance de la classe C]File[/C] par son symbole de classe, alors qu'aucune classe n'existe avec ce symbole.  

{\large {\tt BadFileRead}} : Exception levée lors d'une lecture erronnée dans un fichier de données séquentielles (nombre de champs incorrect, format inattendu, etc.). Pour exploiter cette exception dans le code de l'application, valuer l'argument {\small {\tt file}} (type {\em File*}). Par exemple : [P]BadFileRead((File*)pF);[/P].    

{\large {\tt VoidRecordInFactualDataFile}} : Exception levée lorsqu'on lit un fichier de données factuelles (voir page dédiée) avec une fonction de lecture erronée.  

{\large {\tt UnexpectedEndInclude}} : Exception levée lorsqu'on rencontre, dans un fichier de données en entrée, une directive de fin d'inclusion dans un autre fichier, alors qu'aucune directive d'inclusion n'a été donnée dans un quelconque fichier.  

{\large {\tt UnspecifiedInPath}} : Exception levée lorsqu'on affecte, dans un fichier de données en entrée, un nom simple à un fichier de données, alors qu'on n'a pas spécifié de répertoire pour ce fichier (par [Y]Simulation::SetInDir(char*)[/Y]). 

{\large {\tt UnspecifiedInPath}} : Exception levée lorsqu'on tente de lire un fichier de données séquentielles, alors qu'on n'a pas spécifié le répertoire et le nom simple, ou bien le nom complet, pour ce fichier.  

{\large {\tt ConnexionFailedToDataBase}} : Exception levée lors d'une tentative d'accès à une base de données relationnelle, alors que la connexion n'a pas pu être établie préalablement.
 */
//@{ 
void UnknownFileClassSymbol(char* fileType, UserFileId id);
void UnknownClassSymbol(char* fileType, UserFileId id); // pour compat.
void BadFileOpenMode(File*, int);
void BadFieldValueType(File*, int);
void BadFileName(File*, char*);
void BadFileType(File*, FileClassType);
void BadSearchByFileId(UserFileId);
void BadFileRead(File*);
void BadReadFile(File*);  // pour compat.
void VoidRecordInFactualDataFile(char*);
void UnexpectedEndInclude(char* fileName);
void UnspecifiedInPath(char* simpleName);
void UnspecifiedPath(UserFileId);
																				void ConnexionFailedToDataBase(std::string, std::string);
//@}    

//===========================================================================

/**@name Erreurs de manipulation du chronomètre  

{\large {\tt BadChronometerHandling}} : Exception levée dans un des cas suivants :
\begin{itemize}
\item le chronomètre n'a pas été initialisé (par [Y]InitChrono()[/Y]) avant un [Y]StartChrono()[/Y]
\item le chronomètre n'a pas été initialisé (par [Y]InitChrono()[/Y]) ou déclenché (par [Y]StartChrono()[/Y]) avant un [Y]PauseChrono()[/Y]
\item le chronomètre n'a pas été initialisé (par [Y]InitChrono()[/Y]) avant un [Y]CloseChrono()[/Y]
\end{itemize}

La page 'Fonctions générales de manipulation du temps', documentate l'usage correct des fonctions de chronométrage.

 */
//@{
void BadChronometerHandling(char* service);
//@}   

//===========================================================================

// @Man: Messages d'erreur divers
// @{    

          // Exception levée quand on manipule une date invalide, par exemple antérieure au début de la simulation.
void BadDate(char* fct, int jj, int mm, int yy, int hh, int mn, int ss);
          // Exception levée lorsqu'on construit un intervalle d'entiers avec la première borne supérieure à la seconde.
void BadIntervalBounds(int, int);
          // Exception levée lorsqu'on construit un intervalle de flottants avec la première borne supérieure à la seconde.
void BadIntervalBounds(float, float);
          // Exception levée lorsqu'on construit un intervalle de flottants double précision avec la première borne supérieure à la seconde.
void BadIntervalBounds(double, double);
          // Exception levée sur une erreur de syntaxe dans le fichier des spécifications de sortie.
void BadSyntaxInOSFile(char*, char*);
          // Avertissement émis sur un mauvais format de date sur une spécification de sortie, dans le cas de l'exportation dans une base de données.
void BadSyntaxInDBOSFile();
          // Exception levée sur une obsolescence.
void Obsolescence(char*, char*);
          // Exception levée sur double spécification de sortie sur classes parentes.
void OverSpecificationOnParentClass(char*, char*);
          // Exception levée lorsqu'on manuipule un écart-type non positif.
void NonPositiveStandardDeviation(char*, double);
// @}  

//===========================================================================

// @Man: Messages d'erreur 'moteur'
// @{ 
          // Exception levée lorsqu'on fait référence à une instance d'entité qui n'hérite d'aucune classe de base des entités.
void StrangeEntity(char* service, BasicEntity* pE);
          // Exception levée lorsqu'on fait référence à une instance inexistante d'une classe quelconque. L'argument précise le lieu où la référence a été faite.
void InexistantInstance(char*);
// @}   

//===========================================================================




//@} //fin fonction erreur




#endif  // COREXCEPTION_HH
