//============================================================================
// FILE: C_Strategy.h
//
// TYPE: C++CLASS
// DESCRIPTION: entête des éléments de la stratégie
// 
// AUTHORS: Rellier (INRA/UBIA, Toulouse)
// REFERENCE : "Fondements ontologiques des systèmes pilotés"
//             R. Martin-Clouaire, J.-P. Rellier (INRA/UBIA, Toulouse)
//=X===============X=============================X============================

#ifndef _C_STRATEGY_HH
#define _C_STRATEGY_HH

// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//
#include "C_CoreTypes.h"

// EXTERNAL REFERENCES 


//===========================================================================
//
// STRATEGY
//
//===========================================================================

/**@name Les stratégies de conduite 
\begin{itemize} \item Directive d'inclusion : 
[P]
\#include C_Strategy.h
[/P]
\end{itemize} 
\label{classeStrategy}
Une {\em stratégie de conduite}\index{stratégie de conduite} est un ensemble de spécifications et de règles d'action que suit le gestionnaire (au sens technique) d'un {\em système contrôlé} pour atteindre un certain objectif.\\

La stratégie de conduite du système est, avec la configuration même du système, un des deux éléments de la stratégie (au sens large) du gestionnaire. La configuration du système ne fait donc pas partie de la stratégie de conduite (au sens strict).\\

En effet, la stratégie de conduite est appliquée à un système déjà configuré, au sens où ses éléments et composants sont connus et installés, et où ses propriétés initiales sont fixées. La conduite est adaptée, entre autres choses, à cette configuration. Le raisonnement de configuration du système contrôlé est effectué à un niveau supérieur de la gestion du système ou d'un système englobant. A ce niveau, ce raisonnement peut un être aspect de la stratégie de conduite du système englobant.\\

La notion centrale dans une stratégie de conduite est celle de {\em plan d'activités}\index{plan d'activités}. Ce plan, fixé préalablement, est cependant susceptible de révision ou d'ajustement en cours de conduite. C'est un programme qui génère, répétitivement dans le temps, des alternatives d'actions candidates, parmi lesquelles un choix doit être fait.\\

En conséquence de cela, les trois composants d'une stratégie sont :
\begin{itemize}
\item Un ensemble de {\em blocs activités-ressources}. En première approximation, chaque élément est un {\em plans d'activités} servi par un ensemble propre de ressources.
\item Une {\em trajectoire réactive}, qui est un ensemble de {\em points d'ajustement} de la stratégie elle-même, et plus particulièrement des plans d'activités qu'elle contient.
\item Un ensemble de {\em règles de préférence}. 
\end{itemize}

Dans la version courante de \CD, seul le premier composant est installé sous la forme de classes dédiées. Dans le domaine d'application, les deux autres, au besoin, doivent être programmés en exploitant les services des classes existantes. Des manières de le faire sont présentées ici et là dans la présente documentation.
*/
//@{

/**@name La classe des stratégies de conduite\\

\begin{tabular}{cccll}
{\tiny nom C++} & {\tiny symbole de classe} & {\tiny nom de classe} & {\tiny classe des éléments} & {\tiny classes des composants}\hline
\hline
{\tiny [C]Strategy[/C]} & {\tiny STRATEGY} & {\tiny strategy} & {\tiny } & {\tiny ACTIVITIES_RESOURCES_BLOCK_SET}
\end{tabular} 

On remarque l'absence, à ce stade de développement de \CD, des composants dont les clasess pourraient être REACTIVE_TRAJECTORY et PREFERENCE_RULE_SET.\\

  {\bf Descripteurs :}\\

\begin{tabular}{ccccl}
 & {\tiny type} & {\tiny classe, symbole, nom} & {\tiny v. par défaut} & {\tiny }\hline
\hline
{\tiny C} & {\tiny float} & {\tiny [C]TargetWeight[/C] TARGET_WEIGHT targetWeight} & {\tiny 0.} & {\tiny le poids d'un critère sur un but de type 'cible'}\hline
{\tiny C} & {\tiny float} & {\tiny [C]AntiTargetWeight[/C] ANTI_TARGET_WEIGHT antiTargetWeight} & {\tiny 0.} & {\tiny le poids d'un critère sur un but de type 'anti-cible'}\hline
{\tiny C} & {\tiny float} & {\tiny [C]ActivitySetDefaultUtility[/C] ACTIVITY_SET_DEFAULT_UTILITY activitySetDefaultUtility} & {\tiny 0.} & {\tiny valeur de l'intégrale de Choquet en l'absence de critère d'avaluation}\hline
{\tiny C} & {\tiny float} & {\tiny [C]ActivitySetSatisfactionThreshold[/C] ACTIVITY_SET_SATISFACTION_THRESHOLD activitySetSatisfactionThreshold} & {\tiny 0.} & {\tiny borne inférieure du domaine de satisfaction d'un jeu d'activités}
\end{tabular}

Les descripteurs listés ci-dessus sont liés à l'évaluation du degré de satisfaction d'atteinte de buts attachés à la stratégie. Ils seront justifiés et documentés avec la classe des buts ([C]Goal[/C]) et la classe des critères ([C]Criterion[/C]).

*/
//@{
          ///
//----------------------------------------------------------------------------
class Strategy : public Entity
{
public:

  /**@name LIFECYCLE */
//@{
          // Constructeur de base.
Strategy();
          /// Constructeur par copie profonde. 
Strategy(const Strategy&); 
          /// Destructeur superficiel. 
~Strategy();
          /// Destructeur en profondeur de la stratégie et de ses composants.\index{méthodes!DeleteInDepth (Strategy)}
void DeleteInDepth();
//@}

/**@name  ACCESS */
//@{
           /// Renvoie le système contrôlé par la présente stratégie.\index{méthodes!GetCS -Strategy-}
  ControlledSystem* GetCS();
//@} 

protected: 

private:
};
 
//@}  // fin classe stratégie  


//===========================================================================
//
// ACTIVITIES_RESOURCES_BLOCK
//
//===========================================================================

/**@name Blocs activités-ressources\\

\begin{tabular}{cccll}
{\tiny nom C++} & {\tiny symbole de classe} & {\tiny nom de classe} & {\tiny classe des éléments} & {\tiny classes des composants}\hline
\hline
{\tiny [C]ActivitiesResourcesBlock[/C]} & {\tiny ACTIVITIES_RESOURCES_BLOCK} & {\tiny activitiesResourcesBlock} & {\tiny } & {\tiny ACTIVITY RESOURCE_POOL_CONJUNCTION}
\end{tabular} 

Les deux composants sont :
\begin{itemize}
\item un plan d'activités, instance d'une quelconque des sous-classes de [C]Activity[/C]. On verra dans la page dédiée qu'un plan est un assemblage d'activités dites 'primitives', reliées par des opérateurs.\\
\item un ensemble (plus précisément la réunion) de 'pools de ressources'. Ces pools sont des instances de la classe [C]ResourceSet[/C] (voir la page 'Des ensembles de ressources qui ne sont pas des ressources agrégées').
\end{itemize}

Note importante : On a vu ou on verra qu'une stratégie est un ensemble de "blocs activités-ressources". Les ensembles de ressources associés à ces blocs doivent être disjoints. En d'autres termes, ils constituent une partition de l'ensemble des ressources qui servent l'un ou l'autre des plans de la stratégie. Si, après avoir identifié deux plans, il s'avère qu'une ressource est au service des deux, alors il faut réunir les deux plans. Par exemple, le schéma ci-dessous n'est pas gérable par le moteur de simulation :\\

\begin{verbatim}
[P]strategie : {bloc_1, bloc_2)
bloc 1 : {plan_1, pool_1}
bloc 2 : {plan_2, pool_2}
pool_1 : (r1, r2, r3)
pool_2 : (r1, r4, r5)[/P]
\end{verbatim}

Il doit être remplacé par celui-ci :\\

\begin{verbatim}
[P]strategie : {bloc_1b)
bloc 1b : {plan_1b, pool_1b}
plan_1b : and(plan_1, plan_2)
pool_1b : (r1, r2, r3, r4, r5)[/P]
\end{verbatim}

  {\bf Descripteurs :}\\

\begin{tabular}{ccccl}
 & {\tiny type} & {\tiny classe, symbole, nom} & {\tiny v. par défaut} & {\tiny }\hline
\hline
{\tiny C} & {\tiny Entity*} & {\tiny [C]AttachedEntity[/C] ATTACHED_ENTITY attachedEntity} & {\tiny NULL} & {\tiny l'entité naturellement associée au bloc}
\end{tabular}

Un bloc activités-ressources contient des spécifications d'activités, lesquelles mettent en oeuvre des opérations, lesquelles agissent sur des entités, éléments ou composants du système piloté. On a fréquemment besoin, notamment dans le codage de cet effet des opérations, de pointer sur l'entité touchée. Il est commode de rendre ce pointage relatif à une autre, choisie comme référence pour le plus de pointages possible, et telle que la distance entre cette référence et les objets à pointer soit courte, donc aisée à écrire. Une fois choisie cette entité-référence, on l'affecte comme valeur du descripteur [C]AttachedEntity[/C].\\

Typiquement, cette entité 'attachée' est le système piloté lui-même (la valeur du descripteur [C]AttachedEntity[/C] de l'unique instance de [C]ControlledSystem[/C]). Dans le cas d'une partition du système controlé en sous-systèmes pilotés indépendamment (plans différents servis par des pools de ressources disjoints), les entités attachées aux différents 'blocs activités-ressources' ainsi identifiés seront plus naturellement les sous-systèmes.\\

Voici un exemple de gestion de cette information :

\begin{verbatim}
[P]{
  ...
  Entity* pPlan = ...;                   // le plan d'activités
  Entity* pARB = new ActivitiesResourcesBlock();
  Entity* pPlot = ...;                   // un objet dans le système piloté
  pARB->SetEntityConstValue(ATTACHED_ENTITY, pPlot);
  pPlot->SetIntVarValue(IS_OPERATED, FALSE); // une variable d'état
  pARB->AddComponent(pPlan);
  Entity* pARBSet = pStrategy->GetComponent(ACTIVITIES_RESOURCES_BLOCK_SET);
  pARBSet->AddElement(pARB);
  ...
}
{
  ...
  pInst = ...; // une activité primitive
  Operation* pOt = (Operation*)pInst->GetEntityVarValue(ALLOCATED_OPERATION);
  ActivitiesResourcesBlock* pARB = pOt->GetActivitiesResourcesBlock();
  Entity* pPlot = pARB->GetEntityConstValue(ATTACHED_ENTITY);
  pPlot->SetIntVarValue(IS_OPERATED, TRUE); // var. d'état modifiée par opération
  ...
}[/P]
\end{verbatim}
  
  {\bf Méthodes :}

\begin{itemize}
\item [C]LocalBestActivitySetSelector[/C] sélectionne et renvoie un jeu, interprété ensuite comme le meilleur, parmi un ensemble de jeux d'activités alloués avec succès.
\item [C]LocalCandidateActivitySetSortMethod[/C] modifie l'ordre des activités dans le jeu passé en second argument.
\end{itemize}

\begin{tabular}{ccccl}
{\tiny type} & {\tiny classe, symbole, nom} & {\tiny corps par défaut} & {\tiny classes des arguments}\hline
\hline
{\tiny Entity*} & {\tiny [C]LocalBestActivitySetSelector[/C] LOCAL_BEST_ACTIVITY_SET_SELECTOR localBestActivitySetSelector} & {\tiny} & {\tiny [C]ActivitySetsArgument[C]} \hline
{\tiny Entity*} & {\tiny [C]LocalCandidateActivitySetSortMethod[/C] LOCAL_CANDIDATE_ACTIVITY_SET_SORT_METHOD localCandidateActivitySetSortMethod} & {\tiny } & {\tiny [C]ActivitySetArgument[/C]}
\end{tabular}
  
  {\bf Arguments des méthodes :}

\begin{tabular}{ccccl}
{\tiny type} & {\tiny classe, symbole, nom} & {\tiny v. par défaut} & {\tiny domaine}\hline
\hline
{\tiny Entity*} & {\tiny [C]ActivitiesResourcesBlockArgument[/C] ACTIVITIES_RESOURCES_BLOCK_ARGUMENT activitiesResourcesBlockArgument} & {\tiny NULL} & {\tiny instance de [C]ActivitiesResourcesBlock[/C]} \hline
{\tiny Entity*} & {\tiny [C]ActivitySetsArgument[/C] ACTIVITY_SETS_ARGUMENT activitySetsArgument} & {\tiny NULL} & {\tiny  instance de [C]InstructionSetConjunction[/C]} 
\end{tabular}

Ces méthodes portent une connaissances qui peut aussi être localisée sur le gestionnaire du système (classe [C]Manager[/C]) si elle est reconnue applicable de la même manière sur tous les blocs activités-ressources (et notamment quand il n'y en a qu'un seul). Si une telle connaissance est localisée sur le bloc, elle est exploitée prioritairement (et exclusivement) à celle éventuellement localisée sur le gestionnaire.

Définition de la procédure de sélection du meilleur jeu d'activités :
\begin{verbatim}
[P]BasicEntity* My_BestActivitySetSelector_Body (EntityMethod* pM) {
  Entity* pARB = pM->DescribedEntity();
  Entity* pSetDisjunction = pM->GetEntityArgValue(ACTIVITY_SETS_ARGUMENT);
  int nbSets = pSetDisjunction->GetNumberOfElements();
  BasicEntity* result = NULL;

  for(int i=nbSets-1;i>=0;i--) {
	Entity* pSet_i = pSetDisjunction->GetElement(i);
	Boolean toRemove = FALSE;
    // ...
	}
	if(toRemove) {
	  pSetDisjunction->RemoveElement(i);
	}
  }
  // ...
  if(pSetDisjunction->GetNumberOfElements() > 0) { 
	result = pSetDisjunction->GetElement(0);
  }
  return result;
}[/P]
\end{verbatim}

Assignation de ce corps à la méthode de sélection du meilleur jeu :
\begin{verbatim}
[P]{
  ...
  pInst = ...; // une activité primitive
  Activity* pPlan = pInst->RootActivity();  // le plan d'activités
  ActivitiesResourcesBlock* 
     pARB = (ActivitiesResourcesBlock*)(pPlan->GetSuperEntity());
  pARB->AssignBody(LOCAL_BEST_ACTIVITY_SET_SELECTOR,
				   My_BestActivitySetSelector_Body);
  ...
}[/P]
\end{verbatim}

*/ 
//@{
          ///
class ActivitiesResourcesBlock : public Entity
{
public:
          // Constructeur de base.
   ActivitiesResourcesBlock();
          // Constructeur par copie.
   ActivitiesResourcesBlock(const ActivitiesResourcesBlock&);
          // Destructeur.
   ~ActivitiesResourcesBlock();
 }; 
 
//@}  // fin arb
 
//===========================================================================
//
// ACTIVITIES_RESOURCES_BLOCKSET
//
//===========================================================================

/**@name Ensembles de blocs activités-ressources\\

\begin{tabular}{cccll}
{\tiny nom C++} & {\tiny symbole de classe} & {\tiny nom de classe} & {\tiny classe des éléments} & {\tiny classes des composants}\hline
\hline
{\tiny [C]ActivitiesResourcesBlockSet[/C]} & {\tiny ACTIVITIES_RESOURCES_BLOCK_SET} & {\tiny activitiesResourcesBlockSet} & {\tiny ACTIVITIES_RESOURCES_BLOCK} & {\tiny }
\end{tabular} 

Une instance de cette classe est composant de la stratégie (classe [C]Strategy[/C]) du gestionnaire du système (classe [C]Manager[/C]). Ses éléments sont des instances de [C]ActivitiesResourcesBlock[/C].\\

Cette classe ne possède ni descripteurs ni méthodes propres.
*/ 
//@{
          ///
class ActivitiesResourcesBlockSet : public Entity
{
public:
          // Constructeur de base.
   ActivitiesResourcesBlockSet();
          // Constructeur par copie.
   ActivitiesResourcesBlockSet(const ActivitiesResourcesBlockSet&);
          // Destructeur.
   ~ActivitiesResourcesBlockSet();

/**@name  ACCESS */
//@{
//@}  
 };
 
//@}  // fin arbSet


//@}  // fin stratégies

#endif  // _C_STRATEGY_HH
